#!/usr/bin/python
# -*- coding: utf-8 -*-
##      askwindow.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk
import time

import collect_funcs
import debug
import globs
import gui_funcs
import hostentry
import report
from pref import preferences

_ = gettext.gettext

class AskWindow(debug.Debug) :
	def __init__(self, hosts) :
		debug.Debug.__init__(self)
		self.report = report.Report()
		self.report.prepare(hosts)
		self.user_data = collect_funcs.get_user_data()

	def run(self, window=None) :
		"""
		Shows window of report preparation
		"""
		(text, cid) = self.report.make_short_report()

		self.glade_ask_window = gtk.glade.XML(preferences.opt["glade_ask"])

		self.ask_window = self.glade_ask_window.get_widget("ask_window")
		self.ask_window.connect("delete-event", self.cancel_action)
		if window is not None :
			self.ask_window.set_transient_for(window)

		qicon = self.glade_ask_window.get_widget("quality_icon")
		qicon.set_from_stock(hostentry.q_icons[cid], gtk.ICON_SIZE_SMALL_TOOLBAR)

		sr = self.glade_ask_window.get_widget("short_report")
		bsr = gtk.TextBuffer()
		bsr.set_text(text[:-1])
		sr.set_buffer(bsr)

		sre = self.glade_ask_window.get_widget("speed_report_ex")
		if globs.speedtestnetgui.add_to_report and globs.speedtestnetgui.date is not None:
			sre.show()
			sr = self.glade_ask_window.get_widget("speed_report")
			bsr = gtk.TextBuffer()
			bsr.set_text(globs.speedtestnetgui.get_text_for_report())
			sr.set_buffer(bsr)
		else :
			sre.hide()


		yn = self.glade_ask_window.get_widget("yesno")
		yn.set_active(int(self.user_data["yes"]))
		self.yesno_action(yn)

		nf = self.glade_ask_window.get_widget("name_field")
		nf.set_text(self.user_data["name"])
		pf = self.glade_ask_window.get_widget("phone_field")
		pf.set_text(self.user_data["phone_number"])
		af = self.glade_ask_window.get_widget("address_field")
		af.set_text(self.user_data["address"])

		if preferences.opt["prov_schedule"] != "" :
			cf = self.glade_ask_window.get_widget("convenient_frame")
			cf.set_tooltip_text(_("Our schedule: %s") % preferences.opt["prov_schedule"])

		cal = self.glade_ask_window.get_widget("calendar")
		t = time.localtime()
		cal.select_month(t[1]-1, t[0])
		cal.select_day(t[2])
		self.date_action(cal)

		ttw = self.glade_ask_window.get_widget("time_type")
		ttw.set_active(int(self.user_data["time_type"]))
		self.time_type_action(ttw)

		h1 = self.glade_ask_window.get_widget("hours1")
		h1.set_value(int(self.user_data["time_h1"]))
		m1 = self.glade_ask_window.get_widget("minutes1")
		m1.set_value(int(self.user_data["time_m1"]))
		h2 = self.glade_ask_window.get_widget("hours2")
		h2.set_value(int(self.user_data["time_h2"]))
		m2 = self.glade_ask_window.get_widget("minutes2")
		m2.set_value(int(self.user_data["time_m2"]))

		self.glade_ask_window.signal_connect("on_yesno_changed", self.yesno_action)
		self.glade_ask_window.signal_connect("on_name_field_changed", self.name_field_action)
		self.glade_ask_window.signal_connect("on_phone_field_changed", self.phone_field_action)
		self.glade_ask_window.signal_connect("on_address_field_changed", self.address_field_action)
		self.glade_ask_window.signal_connect("on_date_selected", self.date_action)
		self.glade_ask_window.signal_connect("on_time_type_changed", self.time_type_action)
		self.glade_ask_window.signal_connect("on_hours1_changed", self.hours1_action)
		self.glade_ask_window.signal_connect("on_minutes1_changed", self.minutes1_action)
		self.glade_ask_window.signal_connect("on_hours2_changed", self.hours2_action)
		self.glade_ask_window.signal_connect("on_minutes2_changed", self.minutes2_action)

		self.glade_ask_window.signal_connect("on_cancel_clicked", self.cancel_action)
		self.glade_ask_window.signal_connect("on_save_clicked", self.save_action)
		self.glade_ask_window.signal_connect("on_next_clicked", self.next_action)

	def yesno_action(self, combobox) :
		self.user_data["yes"] = str(combobox.get_active())
		udf = self.glade_ask_window.get_widget("user_data_frame")
		cf = self.glade_ask_window.get_widget("convenient_frame")
		if self.user_data["yes"] == "0" :
			udf.set_sensitive(False)
			cf.set_sensitive(False)
		else :
			udf.set_sensitive(True)
			cf.set_sensitive(True)

	def name_field_action(self, entry) :
		self.user_data["name"] = entry.get_text()

	def phone_field_action(self, entry) :
		self.user_data["phone_number"] = entry.get_text()

	def address_field_action(self, entry) :
		self.user_data["address"] = entry.get_text()

	def date_action(self, calendar) :
		self.user_data["date"] = "%d/%s/%d" % calendar.get_date()

	def time_type_action(self, combobox) :
		self.user_data["time_type"] = str(combobox.get_active())
		t2 = self.glade_ask_window.get_widget("time2")
		if self.user_data["time_type"] == "2" :
			t2.set_sensitive(True)
		else :
			t2.set_sensitive(False)

	def hours1_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			self.user_data["time_h1"] = t

	def minutes1_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			self.user_data["time_m1"] = t

	def hours2_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			self.user_data["time_h2"] = t

	def minutes2_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			self.user_data["time_m2"] = t

	def cancel_action(self, button, event=None) :
		collect_funcs.save_user_data(self.user_data)
		self.ask_window.destroy()

	def save_action(self, button) :
		filename = preferences.const["app_name"] + "-report-" +\
			time.strftime("%Y-%m-%d_%H_%M_%S", preferences.const["begin_date"]) +\
			".txt"
		filename = gui_funcs.show_filechooser_dialog(self.ask_window,
						"Save report...", "save", filename)
		if filename != "" :
			self.report.user_data = self.user_data
			self.report.save(filename, self.ask_window)

	def next_action(self, button) :
		self.cancel_action(button)
		self.report.user_data = self.user_data
		self.report.send(self.ask_window)
