#!/usr/bin/python
# -*- coding: utf-8 -*-
##      autoconfig.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import os
import os.path
import re
import socket
import urllib

import debug
import reg
from pref import preferences

class Autoconfig(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)
		self.hosts = []
		self.hosts_strings = []
		self.resolved = 0
		self.canceled = False

	def get_config(self, url) :
		"""
		This function gets a list of hosts from a url.
		"""
		if url == "" :
			# No error
			return True

		# open remote config
		fhlist = None
		try :
			socket.setdefaulttimeout(preferences.opt["autocfg_timeout"])
			fhlist = urllib.urlopen(url)
			socket.setdefaulttimeout(None)
			self.hosts_strings = fhlist.readlines()
			print "Use remote config"
		except :
			if fhlist :
				fhlist.close()
		else :
			fhlist.close()
			return True

		# if remote config not available 
		fhlist = None
		try :
			fhlist = open(preferences.opt["autocfg_hosts_save"])
			self.hosts_strings = fhlist.readlines()
			print "Use local config"
		except :
			if fhlist :
				fhlist.close()
			return False
		else :
			fhlist.close()
			return True

	def process_config(self):
		"""
		This function processes list of hosts.
		"""
		ip_regexp = "(%s|%s)" % (reg.ip, reg.ip6)
		flag_regexp = "[^\s]+"

		hif_regexp = "^%s\s+%s\s+%s\s*$" % (reg.host, ip_regexp, flag_regexp)
		hi_regexp = "^%s\s+%s\s*$" % (reg.host, ip_regexp)
		hf_regexp = "^%s\s+%s\s*$" % (reg.host, flag_regexp)
		if_regexp = "^%s\s+%s\s*$" % (ip_regexp, flag_regexp)

		self.resolved = 0
		for strg in self.hosts_strings:
			if self.canceled: break;

			# data[i] hasn't white spaces
			data = strg.split()

			if len(data) == 3 :
				# host ip flag
				if re.search(hif_regexp, strg) is not None :
					self.hosts.append((data[0], data[1], data[2]))

			elif len(data) == 2 :
				if re.search(hi_regexp, strg) is not None :
					# host ip
					self.hosts.append((data[0], data[1], "O"))
				elif re.search(if_regexp, strg) is not None:
					# ip flag
					try :
						# ip resolving
						(host, alises, ips) = socket.gethostbyaddr(data[0])
						self.hosts.append((host, ips[0], data[1]))
					except Exception, e :
						print str(e)
						self.hosts.append((data[0], data[0], data[1]))
				elif re.search(hf_regexp, strg) is not None:
					# host flag
					try :
						# host resolving
						(host, alises, ips) = socket.gethostbyaddr(data[0])
						self.hosts.append((host, ips[0], data[1]))
					except Exception, e :
						print str(e)

			elif len(data) == 1 :
				if re.search("^%s$" % ip_regexp, data[0]) is not None :
					# ip
					try :
						# host resolving
						(host, alises, ips) = socket.gethostbyaddr(data[0])
						self.hosts.append((host, ips[0], "O"))
					except Exception, e :
						print str(e)
						self.hosts.append((data[0], data[0], "O"))
				else :
					# host
					try :
						# ip resolving
						(host, alises, ips) = socket.gethostbyaddr(data[0])
						self.hosts.append((host, ips[0], "O"))
					except Exception, e :
						print str(e)
			self.resolved += 1

	def filt_list (self) :
		res = []
		for name, regflag, include in preferences.const["autocfg_include"] :
			if not include :
				continue
			for host, ip, flag in self.hosts :
				if re.search(regflag, flag) is not None :
					res.append((host, ip, flag))
		return res

	def save_list (self) :
		"""
		This function saves a list of hosts to the local file.
		"""
		cache = None
		try :
			if not os.path.exists(preferences.opt["config_user_dir"]) :
				os.makedirs(preferences.opt["config_user_dir"])
			cache = open(preferences.opt["autocfg_hosts_save"], "w")
			for entry in self.hosts :
				cache.write("%s %s %s\n" % entry)
		except :
			print "Can't save config to local file"
		finally :
			if cache :
				cache.close()

	def get_zonen (self, host, ip) :
		""" Results:
				0 -- local address, in zone of the responsibility of ISP;
				1 -- local address, outside a zone of the responsibility of ISP;
				2 -- other hosts (Internet).
		"""
		if not preferences.var["prov_enable_zones"] :
			return 0

		if self.in_hosts(host, ip) or self.in_resp(host, ip) :
			return 0
		elif self.in_local(host, ip) :
			return 1
		return 2

	def in_hosts (self, host, ip) :
		for h,i,f in self.hosts :
			if h == host or i == ip :
				return True
		return False

	def in_resp (self, host, ip) :
		return self.check_addr(host, ip, preferences.opt["prov_resp_hosts"])

	def in_local (self, host, ip) :
		return self.check_addr(host, ip, preferences.opt["prov_local_hosts"])

	def check_addr (self, host, ip, hosts) :
		for regexp in hosts :
			resexpr_h = re.search(regexp, host)
			resexpr_i = re.search(regexp, ip)
			if resexpr_h is not None or\
				resexpr_i is not None :
				return True
		return False

	def get_center_host(self) :
		"""
		This function returns host which was marked as 'S' (center server).
		"""
		center = filter(lambda a : a[2] == "S", self.hosts)
		if center != [] :
			return center[0]
		return ("", "", "")

