#!/usr/bin/python
# -*- coding: utf-8 -*-
##      autoconfiggui.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk
import gtk.glade
import threading
import time

import autoconfig
import debug
import globs
import gui_funcs
from copy import deepcopy
from pref import preferences

_ = gettext.gettext

class AutoProgressBar(autoconfig.Autoconfig) :
	def __init__(self) :
		debug.Debug.__init__(self)
		autoconfig.Autoconfig.__init__(self)
		self.fraction = 0
		self.autolist_error = True
		self.pause = 0.1
		self.hosts = []
		self.reset_rounds()

	def reset_rounds(self) :
		self.first_round  = False
		self.second_round = False

	def get_default_list (self, url, filt):
		"""
		This function runs process of adding hosts to the main table.
		"""
		self.reset_rounds()

		# getting a list of hosts 
		self.autolist_error = not self.get_config(url)
		self.first_round  = True

		# resolving hosts
		if not self.autolist_error :
			self.process_config()
		self.second_round = True

		#
		if filt :
			hosts = self.filt_list()
		else :
			hosts = self.hosts

		center_host = self.get_center_host()[0]
		if center_host != "" :
			preferences.var["center_host"] = center_host

		# adding hosts to the main table
		length = len(hosts)
		i = 0
		for host, ip, flag in hosts:
			if self.canceled : break

			self.label = _("added %(added)d/%(len)d hosts") % \
					{"added" : i, "len" : length}
			self.fraction = float(i) / length

			globs.testing.add_host(ip, flag)
			i += 1

	def run (self, window, url, filt=True) :
		"""
		This function shows configuration's window with progress bar. Progress
		bar displays process of adding hosts to the main table.
		"""
		if url == "" :
			return

		self.canceled = False

		self.label = _("getting list of hosts...")
		# to start configuration in the thread
		th = threading.Thread(target=self.get_default_list, name="get autolist",
						args=(url,filt))
		th.setDaemon(True)
		th.start()

		# to show configuration's window
		gui_funcs.gtk_threads_enter()
		glade_progresswin = gtk.glade.XML(preferences.opt["glade_autoconf"])
		glade_progresswin.signal_connect("on_cancel_clicked",
			self.on_cancel, (url,))

		self.progress_window = glade_progresswin.get_widget("config_progress_window")
		self.progress_window.set_transient_for(window)

		progressbar = glade_progresswin.get_widget("progressbar")
		progressbar.set_text(self.label)
		gui_funcs.gtk_threads_leave()

		# first stage: getting a list of hosts
		while not self.first_round and not self.canceled :
			gui_funcs.gtk_threads_enter()
			progressbar.pulse()
			gui_funcs.gtk_threads_leave()
			time.sleep(self.pause)

		# second stage: resolving hosts's names
		length = len(self.hosts_strings)
		while not self.second_round and not self.canceled :
			self.label = _("resolved %(resd)d/%(len)d hosts") % \
					{"resd" : self.resolved, "len" : length}
			self.fraction = float(self.resolved) / length
			gui_funcs.gtk_threads_enter()
			progressbar.set_text(self.label)
			progressbar.set_fraction(self.fraction)
			gui_funcs.gtk_threads_leave()
			time.sleep(self.pause)

		# third stage: adding hosts
		while th.isAlive() and not self.canceled :
			gui_funcs.gtk_threads_enter()
			progressbar.set_text(self.label)
			progressbar.set_fraction(self.fraction)
			gui_funcs.gtk_threads_leave()
			time.sleep(self.pause)

		# close window
		if url != "" :
			gui_funcs.gtk_threads_enter()
			self.progress_window.destroy()
			gui_funcs.gtk_threads_leave()

		if self.autolist_error and not self.canceled :
			gui_funcs.show_message(window, gtk.MESSAGE_ERROR, _("Error!"),
							_("Can't get configuration"))

	def on_cancel (self, button, url) :
		"""
		This function catchs the mouse click on the cancel button.
		"""
		if url != "" :
			self.progress_window.destroy()
		self.canceled = True

