#!/usr/bin/python
# -*- coding: utf-8 -*-
##      gui_funcs.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gtk
import os
import os.path
import threading

def show_message(window, mtype, name, message) :
	"""
	This function shows the window with the message and ok button.
	"""
	shower = gtk.MessageDialog(window,
		gtk.DIALOG_MODAL,
		mtype,
		gtk.BUTTONS_OK,
		message)
	shower.set_title(name)
	shower.connect("response", lambda a, b : shower.destroy() )

	gtk_threads_enter()
	shower.run()
	gtk_threads_leave()

def show_question(window, message):
	"""
	This function shows the window with the message (question) and yes and no
	buttons.
	"""
	asker = gtk.MessageDialog(parent = window,
					flags = gtk.DIALOG_MODAL,
					buttons = gtk.BUTTONS_YES_NO,
					message_format = message,
					type=gtk.MESSAGE_QUESTION)
	result = (asker.run() == gtk.RESPONSE_YES)
	asker.destroy()
	return result

def show_filechooser_dialog(window, title, action, filename="") :
	"""
	This function shows the file chooser dialog for saving or opening a file.
	"""
	if action == "save" :
		action_gtk = gtk.FILE_CHOOSER_ACTION_SAVE
		button_gtk = gtk.STOCK_SAVE
	elif action == "open" :
		action_gtk = gtk.FILE_CHOOSER_ACTION_OPEN
		button_gtk = gtk.STOCK_OPEN
	else :
		return ""

	fcd = gtk.FileChooserDialog(title, window, action_gtk)
	fcd.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
	fcd.add_button(button_gtk, gtk.RESPONSE_OK)

	if os.name != 'nt' :
		fcd.set_current_folder(os.environ['HOME']) # TODO: last used бы

	fcd.set_property("skip-taskbar-hint", True)
	if filename != "" :
		fcd.set_current_name(filename)
	fcd.set_do_overwrite_confirmation(True)

	gtk_threads_enter()
	response = fcd.run()
	gtk_threads_leave()

	filename = ""
	if response == gtk.RESPONSE_OK :
		filename = fcd.get_filename()

	fcd.destroy()

	return filename

def gtk_iteration() :
	if not is_main_thread() :
		while gtk.events_pending() :
			gtk.main_iteration()

def gtk_threads_enter() :
	if not is_main_thread() :
		gtk.gdk.threads_enter()

def gtk_threads_leave() :
	if not is_main_thread() :
		gtk.gdk.threads_leave()

def get_menu_item (image, label) :
	"""
	This function creates one menu item.
	"""
	mitem = gtk.ImageMenuItem(image)
	mitem.get_children()[0].set_label(label)

	return mitem

def is_main_thread() :
	return threading.currentThread().getName() == "MainThread"

