#!/usr/bin/python
# -*- coding: utf-8 -*-
##      history_graph.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import os
import tempfile

import debug
import open_with
from pref import preferences

class HistoryGraph(debug.Debug) :
	def __init__(self, host) :
		debug.Debug.__init__(self)
		self.data = self.get_data(host)
		self.avg = host.cmsavg()
		self.hostname = host.hostname

	def get_data(self, host) :
		"""
		This function gets the data to plot from the host.
		"""
		data = []
		for secs in host.ping_history :
			if secs < preferences.opt["ping_lat_good"] :
				a = secs
				b = 0
				c = 0
			else :
				a = preferences.opt["ping_lat_good"]
				if secs < preferences.opt["ping_lat_not_so_bad"] :
					b = secs - preferences.opt["ping_lat_good"]
					c = 0
				else :
					b = preferences.opt["ping_lat_not_so_bad"] -\
							preferences.opt["ping_lat_good"]
					c = secs - preferences.opt["ping_lat_not_so_bad"]
			data.append((a,b,c))

		return data

	def prepare(self) :
		"""
		This function prepares configuration file for gnuplot.
		"""
		self.pngfile = tempfile.mktemp() + ".png" # image file
		self.gnuplotfile = tempfile.mktemp() + ".gnuplot" # config for gnuplot
		self.datafile = tempfile.mktemp() + ".data" # file with data

		# files to remove
		preferences.var["files_to_remove"].append(self.pngfile)
		preferences.var["files_to_remove"].append(self.gnuplotfile)
		preferences.var["files_to_remove"].append(self.datafile)

		# to save the data to the data file
		f = open(self.datafile, 'w')
		for d1,d2,d3 in self.data :
			f.write("%f %f %f\n" % (d1,d2,d3))
		f.close()

		# to save configuration file
		# TODO: translatable labels.
		# but gnuplot not supports utf8 with default font
		f = open(self.gnuplotfile, 'w')
		#f.write("set terminal png font '%s' 10\n" % preferences.var["font"])
		f.write("set terminal png\n")
		f.write("set output '%s'\n" % self.pngfile)
		f.write("set style data histogram\n")
		f.write("set style histogram rowstacked\n")
		f.write("set xlabel 'number of ping'\n")
		f.write("set ylabel 'latency (ms)'\n")
		f.write("set xrange [0:%d]\n" % len(self.data))
		f.write("set yrange [0:*]\n")
		f.write("set style fill solid 0.7\n")
		f.write("set boxwidth 1\n")
		f.write("set title 'History graph for %s'\n" % self.hostname)
		f.write(
			("plot '%s' " % self.datafile)  + \
			"   using 1 title '' lt 2," + \
			"'' using 2 title '' lt 7," + \
			"'' using 3 title '' lt 1," + \
			"%f title 'average' lt 3\n" % self.avg
		)

	def create(self) :
		"""
		This function creates graph.
		"""
		self.prepare()
		os.system("gnuplot %s" % self.gnuplotfile)

	def run(self) :
		"""
		This function creates and shows graph.
		"""
		self.create()
		open_with.open_link(self.pngfile, preferences.var["open_image"])

