#!/usr/bin/python
# -*- coding: utf-8 -*-
##      hostentry.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gtk

import collect_funcs
import debug
from pref import preferences

q_serious_icon = gtk.STOCK_DIALOG_ERROR
q_minor_icon   = gtk.STOCK_DIALOG_WARNING
#q_good_icon	= gtk.STOCK_DIALOG_INFO
q_good_icon	= gtk.STOCK_APPLY
q_unknown_icon = gtk.STOCK_DIALOG_QUESTION

q_icons = [q_serious_icon, q_minor_icon, q_good_icon, q_unknown_icon]

class HostEntry(debug.Debug) :
	def __init__(self, hostname="localhost", ip="127.0.0.1") :
		debug.Debug.__init__(self)

		self.hostname = hostname # name of host
		self.ip = ip			 # ip of host
		self.country_code = None # country code
		self.country_name = None # country name
		self.lpath = []		  # length of a path
		self.out = 0			 # pings send
		self.loss = 0			# pings lost
		self.msmin = -1		  # min delay
		self.msmax = -1		  # max delay
		self.mssum = 0		   # sum of delays
		self.avail = False	   # True if host available with traceroute
		self.previous = ""	   # previous node
		self.zone = 0			# zone of responsibility
		self.flag = ""		   # flag for report
		self.traceout = ""	   # output of traceroute
		self.enable = True	   # test or not
		self.ping_history = []   # list of latencies

		self.sep = " "

	def __str__(self) :
		res = self.hostname + " " + self.ip
		if self.flag != "" :
			res += " " + self.flag

		return res

	def __eq__(self, hostentry) :
		return (self.hostname == hostentry.hostname and
				self.ip == hostentry.ip)

	def clossper(self) :
		"""
		This function calculates percent of packages which was lost.
		"""
		if self.out != 0 :
			return 100*float(self.loss)/float(self.out)
		else :
			return 0

	def cmsavg(self) :
		"""
		This function calculates average of the latencies.
		"""
		n = self.out - self.loss
		if n != 0 :
			return float(self.mssum)/float(n)
		else :
			return 0

	def defaults(self) :
		"""
		This function resets all counters.
		"""
		self.out = 0
		self.loss = 0
		self.msmin = -1
		self.msmax = -1
		self.mssum = 0
		del self.ping_history
		self.ping_history = []

		return self

	def get_quality(self) :
		"""
		This function estimates quality of the connection to the host.
		"""
		if self.out == 0 :
			return len(q_icons) - 1

		per = self.clossper()
		latavg = self.cmsavg()
		ll = collect_funcs.estimate(per, 0,
			preferences.opt["ping_loss_good"],
			preferences.opt["ping_loss_not_so_bad"])
		lm = collect_funcs.estimate(latavg, 0,
			preferences.opt["ping_lat_good"],
			preferences.opt["ping_lat_not_so_bad"])

		return min(lm, ll)

