#!/usr/bin/python
# -*- coding: utf-8 -*-
##      mailer.py
##
##
## Copyright (C) 2009 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import smtplib
import urllib
from email.MIMEText import MIMEText

import debug
import open_with
from pref import preferences

class Mailer(debug.Debug) :
	# NOTE: the subject only in ascii 
	def __init__(self) :
		debug.Debug.__init__(self)
		self.subj = ""
		self.body = ""

	def send(self) :
		"""
		This function sends a mail.  It uses method from the preferences.
		"""
		if preferences.opt["mail_method"] == "mailer" :
			self.send_mailer()
		elif preferences.opt["mail_method"] == "mta" :
			self.send_MTA()
		elif preferences.opt["mail_method"] == "direct" :
			self.send_direct()
		else :
			raise Exception(_("Mail method is unknown"))

	def send_mailer(self) :
		"""
		This function opens a default mailer with a mail.
		"""
		link = "mailto:" + preferences.opt["prov_maintenance_crew_email"] +\
			"?Subject=%s" % self.subj +\
			"&Body=%s" % urllib.quote(self.body)

		if not open_with.open_link(link, preferences.var["open_mail"]) :
			raise Exception(_("Can't open the mailer: ") + \
					preferences.var["open_mail"])

	def send_MTA(self) :
		"""
		This function sends a mail.  It uses the MTA.
		"""
		self.send_smtp()

	def send_direct(self) :
		"""
		This function sends a mail.  It uses the smtp server from the
		preferences.
		"""
		if preferences.opt["mail_server"] == "" :
			raise Exception(_("Address of a SMTP server is not set"))

		self.send_smtp(preferences.opt["mail_server"])

	def send_smtp(self, server=None) :
		"""
		This function sends smtp mail.
		"""
		(me, to, msg) = self.create_mime()

		s = smtplib.SMTP()
		if server is None :
			s.connect()
		else :
			s.connect(server)

		s.sendmail(me, [to], msg.as_string())
		s.close()

	def create_mime(self) :
		"""
		This function creates a mail.
		"""
		me = preferences.opt["prov_subscriber_email"]
		to = preferences.opt["prov_maintenance_crew_email"]

		enc = preferences.opt["encoding_report_mail"]
		if enc == '' :
			enc = 'UTF-8'

		msg = MIMEText('', _charset=enc)
		del msg['Content-Transfer-Encoding']
		msg['Content-Transfer-Encoding'] = '8bit'
		msg.set_payload(self.body)
		msg['Subject'] = self.subj
		msg['From'] = me
		msg['To'] = to

		return (me, to, msg)

