#!/usr/bin/python
# -*- coding: utf-8 -*-
##      netmap.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import string
import tempfile
from math import log

import debug
import open_with
from pref import preferences

try :
	import pydot
	preferences.var["pydot_exist"] = True
except ImportError:
	preferences.var["pydot_exist"] = False

class Map(debug.Debug) :
	def __init__(self, hosts) :
		debug.Debug.__init__(self)
		self.hosts = hosts

	def prepare(self) :
		"""
		This function prepare the network graph.
		"""
		# blank fields around graph
		self.netmap = pydot.Dot(pad="2")

		#if not pydot.__version__.startswith('0.') :
		#	self.netmap.set_node_defaults(fontname="sans")
		#	self.netmap.set_node_defaults(fontcolor="black")
		#	self.netmap.set_node_defaults(fontsize="10")
		#	self.netmap.set_node_defaults(shape="circle")
		#	self.netmap.set_node_defaults(style="filled")
		#	self.netmap.set_node_defaults(width="0.3")
		#	self.netmap.set_node_defaults(height="0.3")
		#	self.netmap.set_node_defaults(fixedsize="true")
		#	self.netmap.set_node_defaults(color="gray")
		#
		#	self.netmap.set_edge_defaults(fontname="sans")
		#	self.netmap.set_edge_defaults(fontsize="10")
		#	self.netmap.set_edge_defaults(fontcolor="black")
		#	self.netmap.set_edge_defaults(color="grey")
		#	self.netmap.set_edge_defaults(arrowtail="false")
		#	self.netmap.set_edge_defaults(arrowhead="false")

		# person's computer
		n = pydot.Node("you")
		n.set_label("you")
		#if pydot.__version__.startswith('0.') :
		self.node_set_attrs(n)
		n.set_color("red")
		self.netmap.add_node(n)

		tr = string.maketrans("-.:", "___")
		for host_entry in self.hosts :
			# to add the host
			node_name = (host_entry.hostname + host_entry.ip).translate(tr)
			prev_node_name = host_entry.previous.translate(tr)

			n = pydot.Node(node_name)
			#if pydot.__version__.startswith('0.') :
			self.node_set_attrs(n)

			# to add the edge to a parent for this host
			e = pydot.Edge(prev_node_name, node_name)
			#if pydot.__version__.startswith('0.') :
			e.set_fontname(preferences.var["font"])
			e.set_fontsize("10")
			e.set_fontcolor("black")
			e.set_color("grey")
			e.set_arrowtail("false")
			e.set_arrowhead("false")

			# to calculate the edge length and the edge style.
			# if host unavailable : style = dashed
			# else				: style = line
			thop = host_entry.cmsavg()
			avail = host_entry.avail

			label = host_entry.hostname
			if host_entry.country_name is not None :
				label += ", %s" % host_entry.country_name
			if thop != 0 :
				label += " (%.2f)" % thop
			n.set_label(label)

			if not avail :
				e.set_style("dashed")

			e.set_len("%f" % self.time_to_len(thop))

			# to add the node and the edge to the network graph
			self.netmap.add_node(n)
			self.netmap.add_edge(e)

	def node_set_attrs(self, node) :
		node.set_fontname(preferences.var["font"])
		node.set_fontcolor("black")
		node.set_fontsize("10")
		node.set_shape("circle")
		node.set_style("filled")
		node.set_width("0.3")
		node.set_height("0.3")
		node.set_fixedsize("true")
		node.set_color("gray")

	def time_to_len(self, time) :
		"""
		This function calculates edges length.
		"""
		if time < 0.25 :
			time = 0.25
		return (5 + 1.2*log(time)) * 0.3285;

	def create(self) :
		"""
		This function creates the image file.
		"""
		self.prepare()
		filename = tempfile.mktemp()
		self.pngfile = filename + ".png"
		preferences.var["files_to_remove"].append(self.pngfile)
		self.netmap.write(self.pngfile, prog="neato", format="png")

	def run (self) :
		"""
		This function creates and shows the image file.
		"""
		self.create()
		open_with.open_link(self.pngfile, preferences.var["open_image"])

