#!/usr/bin/python
# -*- coding: utf-8 -*-
##      nmap.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import os
import re
import subprocess

import debug
import reg

class NMap(debug.Debug) :
	def __init__ (self, network) :
		debug.Debug.__init__(self)
		self.nmapexpr = "^Host (%s|%s) .* up .*" % (reg.ip, reg.ip6)
		self.network = network

	def get_cmd(self):
		opt_ipv6 = ""
		if ':' in self.network and re.search("^%s$" % reg.ip6, self.network) :
			# probably IPv6
			opt_ipv6 = "-6"
		return  "nmap %s -sP -n \"%s\"" % (opt_ipv6, self.network)

	def get_available (self) :
		"""
		This function gets available hosts from a network (for example,
		192.168.0.0/24 or 192.168.0.0-255).  It returns list of ip addresses.
		"""
		available_hosts = []

		p = subprocess.Popen(self.get_cmd(), shell=True, stdout=subprocess.PIPE)
		sts = os.waitpid(p.pid, 0)
		if sts[1] != 0 :
			return None
		out = p.stdout
		for strg in out :
			m = re.match(self.nmapexpr, strg)
			if m is not None :
				available_hosts.append(m.group(1))
		out.close()

		return available_hosts

