#!/usr/bin/python
# -*- coding: utf-8 -*-
##      ping.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import math
import os
import re

import debug
import reg
from pref import preferences

class Ping(debug.Debug) :
	def __init__ (self, host) :
		debug.Debug.__init__(self)
		if os.name == 'nt' :
			self.pingexpr = ":\s*.*=\d+\s*.*[=<>](\d+\.?\d*).*=\d+"
		else :
			self.pingexpr = "(rtt)? min/avg/max(/mdev)? = (\d+\.?\d*)/.*"

		self.host = host
		self.size = preferences.opt["ping_size"]
		self.timeout = preferences.opt["ping_timeout"]
		if not preferences.var["fping_exist"] :
			self.timeout = int(math.ceil(float(self.timeout) / 1000))

	def get_cmd(self):
		"""
		This function returns the command for one ping request.
		"""
		cmd = "ping"
		if preferences.var["fping_exist"] : cmd = "fping"

		if ':' in self.host and re.search("^%s$" % reg.ip6, self.host) :
			# probably IPv6
			cmd = "ping6"
			if preferences.var["fping_exist"] : cmd = "fping6"

		if os.name == 'nt' :
			return  '%s -n 1 -l "%d" -w "%d" "%s"' %\
						(cmd, self.size, self.timeout, self.host)
		else :
			if preferences.var["fping_exist"] :
				return  '%s -c 1 -b "%d" -t "%d" "%s" 2>&1' %\
						(cmd, self.size, self.timeout, self.host)
			else :
				return  '%s -c 1 -s "%d" -W "%d" "%s"' %\
						(cmd, self.size, self.timeout, self.host)

	def ping (self) :
		"""
		This function makes one ping request and returns latency in ms. It
		returns -1 when the ping request lost.
		"""
		pingres = "";

		pingout = os.popen(self.get_cmd())
		for strg in pingout :
			pingres += strg
		pingout.close()

		resexpr = re.search(self.pingexpr, pingres)
		if resexpr is not None :
			return float(resexpr.group(3))

		return -1

