#!/usr/bin/python
# -*- coding: utf-8 -*-
##      pref.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import copy
import gettext
import gtk
import gtk.glade
import locale
import os
import re
import socket
import sys
import time

import reg
from config import readconfig, saveconfig

_ = gettext.gettext

class Pref() :
	def __init__(self) :
		"""
		This function initializes the default parameters.
		"""
		self.opt = {}   # for user options
		self.const = {} # for constants
		self.var = {}   # for temprary variables
		# app
		self.const["app_name"] = "gnetworktester"

		self.opt["locale_path"] = "../../locale"
		if locale.getdefaultlocale() != (None, None) :
			gettext.bindtextdomain(self.const["app_name"],
				self.opt["locale_path"])
			gettext.textdomain(self.const["app_name"])
			gtk.glade.bindtextdomain(self.const["app_name"],
				self.opt["locale_path"])
			gtk.glade.textdomain(self.const["app_name"])

		self.const["app_version"] = "0.11.1"
		self.opt["report_version"] = ""
		self.const["app_email"] = "ms@pereslavl.ru"
		self.const["app_descr"] = _(\
"Testing of stability and quality of network connection. \
%s application helps users in testing the quality of network \
connection and in sending complaints on that to provider.") % self.const["app_name"]
		self.const["app_url"] = "http://sourceforge.net/projects/" + self.const["app_name"]
		self.const["app_authors"] = [
						_("Developers:"),
						"Max V. Stotsky (lollo) <ms@pereslavl.ru>",
						"",
						_("Past Developers:"),
						"Veniamin M. Kasimov <mamedich@pereslavl.ru>",
						"",
						_("Thanks:"),
						"Botik Lab.",
						"Anton Kuznetsov",
		]
		self.const["app_translators"] = "Nadya Zhivchikova <ming@pereslavl.ru>\n"
		self.const["app_license"] = _(\
"""This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; version 3 or later.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.""")

		self.const["begin_date"] = time.localtime()
		self.const["end_date"] = time.localtime(0)

		# paths
		self.opt["glade_path"] = "../data/glade"
		self.opt["glade_main"] = self.opt["glade_path"] + "/main_window.glade"
		self.opt["glade_autoconf"] = self.opt["glade_path"] + "/autoconfig.glade"
		self.opt["glade_ask"] = self.opt["glade_path"] + "/ask.glade"
		self.opt["glade_pref"] = self.opt["glade_path"] + "/preferences.glade"
		self.opt["glade_speed"] = self.opt["glade_path"] + "/speed.glade"
		self.opt["glade_speedtestnet"] = self.opt["glade_path"] + "/speedtestnet.glade"

		self.opt["app_logo"] = "../data/pixmaps/%s.svg" % self.const["app_name"]
		self.opt["app_icon"] = "../../pixmaps/%s.svg" % self.const["app_name"]
		self.opt["app_icon_warn"] = "../data/pixmaps/%s_icon_warn.svg" % self.const["app_name"]
		self.opt["app_icon_err"] = "../data/pixmaps/%s_icon_err.svg" % self.const["app_name"]
		if not os.path.exists(self.opt["app_logo"]) :
			self.opt["app_logo"] = "../" + self.const["app_name"] + ".svg"
			self.opt["app_icon"] = "../" + self.const["app_name"] + "_icon.svg"

		self.opt["config_system"] = "../../../../etc/"+self.const["app_name"]+".cfg"
		if not os.path.exists(self.opt["config_system"]) :
			self.opt["config_system"] = "../"+self.const["app_name"]+".cfg"

		if os.name == 'nt' :
			self.opt["config_user_dir"] = os.environ["APPDATA"] + "/" + self.const["app_name"]
		else :
			self.opt["config_user_dir"] = os.environ["HOME"] + "/.config/" + self.const["app_name"]

		self.opt["config_user"] = self.opt["config_user_dir"] + "/"+self.const["app_name"]+".cfg"
		self.opt["config_user_data"] = self.opt["config_user_dir"] + "/user_data"
		self.opt["autocfg_hosts_save"] = self.opt["config_user_dir"] + "/hosts.save"
		self.opt["host_entry_history_file"] = self.opt["config_user_dir"] + "/host.history"
		# ping
		self.opt["ping_number_of_tests"] = 100
		self.opt["ping_pause"] = 50
		self.opt["ping_size"] = 1450
		# auto configurations
		self.opt["autocfg_hosts_url"] = ""
		self.opt["autocfg_timeout"] = 10
		self.const["autocfg_include"] = [
								("box",	 "^B", False),
								("router",  "^R", True),
								("main",	"^S", True),
								("gateway", "^G", True),
								("other",   "^O", False),
						]
		# criterion
		## (-∞, excel]		 -- excellent
		## [excel, good]	   -- good
		## [good, not_so_bad] -- not so bad
		## [not_so_bad, ∞)	-- very bad
		## потери
		self.opt["ping_loss_excel"] = 2
		self.opt["ping_loss_good"] = 5
		self.opt["ping_loss_not_so_bad"] = 10
		self.opt["cable"] = True
		## latency
		if self.opt["cable"] :
			self.opt["ping_timeout"] = 250 # ms
			self.opt["ping_lat_excel"] = 30
			self.opt["ping_lat_good"] = 100
			self.opt["ping_lat_not_so_bad"] = 200
		else :
			self.opt["ping_timeout"] = 3000 # ms
			self.opt["ping_lat_excel"] = 600
			self.opt["ping_lat_good"] = 2000
			self.opt["ping_lat_not_so_bad"] = 5000
		## number of teste
		self.opt["ping_out_excel"] = -999
		self.opt["ping_out_good"] = -99
		self.opt["ping_out_not_so_bad"] = -29
		## packet's size
		self.opt["ping_psize_excel"] = -1449
		self.opt["ping_psize_good"] = -1449
		self.opt["ping_psize_not_so_bad"] = -64

		# to get hostname
		delim = "."
		try :
			domain = socket.getfqdn().split('.', 1)[1]
			if domain == "localdomain" :
				delim = domain = ""
		except IndexError :
			delim = domain = ""

		self.opt["host"] = socket.gethostname()
		self.opt["domain"] = domain
		self.opt["hostname"] = self.opt["host"] + delim + self.opt["domain"]

		# to get ip
		try :
			ips = socket.gethostbyname_ex(self.opt["hostname"])[2]
		except socket.gaierror:
			ips = []

		ips_f = filter(lambda x : x not in ["127.0.0.1", "::1"], ips)
		if ips == [] :
			self.opt["ip"] = "127.0.0.1"
		elif ips_f == [] :
			self.opt["ip"] = ips[0]
		else :
			self.opt["ip"] = ips_f[0]

		# prov
		self.opt["prov_name"] = ""
		self.opt["prov_subscriber_name"] = ""
		self.opt["prov_subscriber_email"] = "postmaster@" + self.opt["hostname"]
		self.opt["prov_maintenance_crew_email"] = ""
		self.opt["prov_schedule"] = ""
		self.opt["prov_resp_hosts"] = []
		self.opt["prov_local_hosts"] = []
		self.var["prov_enable_zones"] = True
		self.const["prov_zones"] = [
			(
				_("in zone of the responsibility of %s"),
				"",
				"../data/pixmaps/in_resp.svg",
			),
			(
				_("outside a zone of the responsibility of %s"),
				_(" (computers of subscribers -- they may be out of order)"),
				"../data/pixmaps/out_resp.svg",
			),
			(
				_("in Internet"),
				_(" (outside a zone of the responsibility of %s)"),
				"../data/pixmaps/world.svg",
			),
		]

		self.opt["speed_gateway"] = self.find_gateway()
		self.opt["speed_discard_port"] = 9
		self.opt["speed_size1"] = 50*1024	 # 50 KiB
		self.opt["speed_size2"] = 1*1024*1024 # 1  MiB
		self.opt["speed_proto"] = "discard" # speed to the report "discard" always

		self.opt["open_with"] = "auto" # auto or custom
		self.opt["open_mail_custom"] = "thunderbird"
		self.opt["open_image_custom"] = "geeqie"
		self.opt["open_browser_custom"] = "firefox"
		self.var["open_mail"] = ""
		self.var["open_image"] = ""
		self.var["open_browser"] = ""
		self.opt["open_auto"] = "xdg-open"

		self.opt["mail_method"] = "mailer" # mailer, mta or direct
		self.opt["mail_server"] = ""	   # adress of smtp server

		self.opt["show_traceroute_result"] = False

		self.opt["include_about_os"] = True
		self.opt["include_about_interfaces"] = True
		self.opt["include_rubbish"] = True

		self.opt["encoding_report_mail"] = ""
		self.opt["encoding_report_file"] = ""
		self.opt["encoding_userwish"] = ""

		self.opt["host_entry_history_length"] = 30

		self.opt["debug"] = False
		self.var["debug"] = False

		self.var["files_to_remove"] = []
		self.var["font"] = "sans"

		self.var["center_host"] = ""

		self.opt["speedtest_icon"] = "../data/pixmaps/speedtest_net.png"
		self.opt["speedtest_config_url"] = "http://www.speedtest.net/speedtest-config.php"
		self.opt["speedtest_latency_path"] = "/latency.txt"
		self.opt["speedtest_down_data_path"] = "/random500x500.jpg"
		self.opt["speedtest_up_data_size"] = 512000 # bytes
		self.opt["speedtest_latency_attempts"] = 10
		self.opt["speedtest_down_attempts"] = 3
		self.opt["speedtest_up_attempts"] = 3

		self.const["earth_radius"] = 6371.0
		self.opt["earth_flags"] = "../data/pixmaps/flags"

		self.opt["googlemap_url"] = "http://gnetworktester.sourceforge.net/cgi-bin/google.cgi?"
		self.opt["google_icon"] = "../data/pixmaps/google.png"

		self.check_soft("nmap")
		self.check_soft("gnuplot")
		self.check_soft("fping")

		#for key in self.opt.keys() :
		#	print key, self.opt[key]

	def read_system(self) :
		"""
		This function reads the system configuration file
		('/etc/gnetworktester.cfg' by default)
		"""
		readconfig(self.opt["config_system"], self.opt)

	def read_user(self) :
		"""
		This function reads the user configuration file
		('$HOME/.config/gnetworktester/gnetworktester.cfg' by default).
		"""
		readconfig(self.opt["config_user"], self.opt)

	def save_user(self) :
		"""
		This function saves difference between the system configuration and the
		user configuration to the user configuration file.
		"""
		opt_diff = {}
		for key in self.opt :
			if self.opt[key] != self.opt_backup[key] :
				opt_diff[key] = self.opt[key]

		if not os.path.exists(preferences.opt["config_user_dir"]) :
			os.makedirs(preferences.opt["config_user_dir"])

		saveconfig(self.opt["config_user"], opt_diff)

	def update_open_with(self) :
		"""
		This function sets application for a mailto links and a png files.
		"""
		if self.opt["open_with"] == "custom" :
			self.var["open_mail"] = self.opt["open_mail_custom"]
			self.var["open_image"] = self.opt["open_image_custom"]
			self.var["open_browser"] = self.opt["open_browser_custom"]
		else :
			self.var["open_mail"] = self.opt["open_auto"]
			self.var["open_image"] = self.opt["open_auto"]
			self.var["open_browser"] = self.opt["open_auto"]

	def update_enable_zones(self) :
		if self.opt["prov_resp_hosts"] == [] :
			self.var["prov_enable_zones"] = False

	def make_checkpoint(self) :
		"""
		This function saves a backup of the system configurations.
		"""
		self.opt_backup = copy.deepcopy(self.opt)

	def find_gateway(self) :
		"""
		This function gets network getway.
		"""
		if os.name == 'nt' :
			f = os.popen("route print")
		else :
			f = os.popen("netstat -rn")

		if f == None :
			print "Error: can't execute netstat"
			sys.exit()

		if os.name == 'nt' :
			reg = "^(\s+0\.0\.0\.0)\s+\s+0\.0\.0\.0\s+(\d+\.\d+\.\d+\.\d+)\s.*$"
		else :
			reg = "^(0\.0\.0\.0|default)\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"
		for strg in f :
			ip = re.search(reg, strg)
			if ip != None :
				f.close()
				return ip.group(2)
		f.close()
		print "No gateway found in routing table."
		sys.exit()

	def check_soft(self, prog) :
		res = os.system("which %s >%s" % (prog, os.devnull))
		self.var["%s_exist" % prog] = res == 0

preferences = Pref()
preferences.read_system()
preferences.make_checkpoint()
preferences.read_user()
preferences.update_open_with()
preferences.update_enable_zones()

