#!/usr/bin/python
# -*- coding: utf-8 -*-
##      prefwindow.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk
import re
import string
from functools import partial

import config
import debug
from pref import preferences

_ = gettext.gettext

class PrefWindow(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)

	def run(self, window) :
		"""
		This function shows preference's window.
		"""
		self.glade_pref_window = gtk.glade.XML(preferences.opt["glade_pref"])

		self.pref_window = self.glade_pref_window.get_widget("pref_window")
		self.pref_window.connect("delete-event", self.close_action)
		if window is not None :
			self.pref_window.set_transient_for(window)

		self.cat_create()

		# first tab
		## name of the ISP
		sn = self.glade_pref_window.get_widget("isp_name")
		sn.set_text(preferences.opt["prov_name"])

		## email of the maintenance crew
		se = self.glade_pref_window.get_widget("isp_email")
		se.set_text(preferences.opt["prov_maintenance_crew_email"])

		## name of the user connection
		sn = self.glade_pref_window.get_widget("subscr_name")
		sn.set_text(preferences.opt["prov_subscriber_name"])

		## email of the user
		se = self.glade_pref_window.get_widget("subscr_email")
		se.set_text(preferences.opt["prov_subscriber_email"])

		## modem or cable?
		rb_name = "modem"
		if preferences.opt["cable"] :
			rb_name = "cable"
		rb = self.glade_pref_window.get_widget(rb_name)
		rb.set_active(True)

		## how much ping tests?
		tn = self.glade_pref_window.get_widget("tests_number")
		tn.set_value(preferences.opt["ping_number_of_tests"])

		## how big ping packets?
		ts = self.glade_pref_window.get_widget("tests_size")
		ts.set_value(preferences.opt["ping_size"])

		## how long pause between pings?
		tp = self.glade_pref_window.get_widget("tests_pause")
		tp.set_value(preferences.opt["ping_pause"])

		## do you want to see result of the trace routing?
		st = self.glade_pref_window.get_widget("show_trace")
		st.set_active(preferences.opt["show_traceroute_result"])

		# second tab
		## default or custom applications?
		index = 0
		if preferences.opt["open_with"] == "custom" :
			index = 1

		ac = self.glade_pref_window.get_widget("applications_combobox")
		ac.set_active(index)

		## or maybe do you want to use the custom applications?
		cie = self.glade_pref_window.get_widget("custom_imager_entry")
		cie.set_text(preferences.opt["open_image_custom"])
		cme = self.glade_pref_window.get_widget("custom_mailer_entry")
		cme.set_text(preferences.opt["open_mail_custom"])
		cbe = self.glade_pref_window.get_widget("custom_browser_entry")
		cbe.set_text(preferences.opt["open_browser_custom"])

		self.applications_action(ac)

		## How do you want to send the emails?
		self.mail_methods = ["mailer", "mta", "direct"]
		index = self.mail_methods.index(preferences.opt["mail_method"])
		emc = self.glade_pref_window.get_widget("email_methods_combobox")
		emc.set_active(index)

		self.email_methods_action(emc)

		# third tab
		self.glade_pref_window.get_widget("warning_frame").show()
		self.glade_pref_window.get_widget("options_filter").hide()
		self.glade_pref_window.get_widget("options_table").hide()

		# actions
		self.glade_pref_window.signal_connect("on_cat_table_event", self.cat_table_action)

		self.glade_pref_window.signal_connect("on_isp_name_changed", self.isp_name_action)
		self.glade_pref_window.signal_connect("on_isp_email_changed", self.isp_email_action)
		self.glade_pref_window.signal_connect("on_subscr_name_changed", self.subscr_name_action)
		self.glade_pref_window.signal_connect("on_subscr_email_changed", self.subscr_email_action)
		self.glade_pref_window.signal_connect("on_modem_clicked", self.modem_action)
		self.glade_pref_window.signal_connect("on_cable_clicked", self.cable_action)
		self.glade_pref_window.signal_connect("on_tests_number_changed", self.test_number_action)
		self.glade_pref_window.signal_connect("on_tests_size_changed", self.test_size_action)
		self.glade_pref_window.signal_connect("on_tests_pause_changed", self.test_pause_action)
		self.glade_pref_window.signal_connect("on_show_trace_clicked", self.show_trace_action)

		self.glade_pref_window.signal_connect("on_applications_changed", self.applications_action)
		self.glade_pref_window.signal_connect("on_custom_imager_changed", self.imager_action)
		self.glade_pref_window.signal_connect("on_custom_mailer_changed", self.mailer_action)
		self.glade_pref_window.signal_connect("on_custom_browser_changed", self.browser_action)
		self.glade_pref_window.signal_connect("on_email_methods_changed", self.email_methods_action)
		self.glade_pref_window.signal_connect("on_server_entry_changed", self.server_entry_action)

		self.glade_pref_window.signal_connect("on_promise_clicked", self.promise_action)
		self.glade_pref_window.signal_connect("on_options_filter_changed", self.options_filter_action)

		self.glade_pref_window.signal_connect("on_close_clicked", self.close_action)

	def cat_table_action(self, treeview, event) :
		selection = treeview.get_selection()
		(treemodel, treeiter) = selection.get_selected()
		if treeiter is None : # Nothing selected
			selection.select_path(0)
			path = 0
		else :
			(path, ) = treemodel.get_path(treeiter)

		pref_notebook = self.glade_pref_window.get_widget("pref_notebook")
		pref_notebook.set_current_page(path)

	def isp_name_action(self, entry) :
		preferences.opt["prov_name"] = entry.get_text()

	def isp_email_action(self, entry) :
		preferences.opt["prov_maintenance_crew_email"] = entry.get_text()

	def subscr_name_action(self, entry) :
		preferences.opt["prov_subscriber_name"] = entry.get_text()

	def subscr_email_action(self, entry) :
		preferences.opt["prov_subscriber_email"] = entry.get_text()

	def modem_action(self, radiobutton) :
		preferences.opt["cable"] = False

	def cable_action(self, radiobutton) :
		preferences.opt["cable"] = True

	def test_number_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			preferences.opt["ping_number_of_tests"] = int(t)

	def test_size_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			preferences.opt["ping_size"] = int(t)

	def test_pause_action(self, spinbutton) :
		t = spinbutton.get_text()
		if t != "" :
			preferences.opt["ping_pause"] = int(t)

	def show_trace_action(self, checkbutton) :
		preferences.opt["show_traceroute_result"] = checkbutton.get_active()

	def applications_action(self, combobox) :
		index = combobox.get_active()
		cap = self.glade_pref_window.get_widget("custom_apps_frame")
		if index == 1 :
			#cap.show()
			cap.set_sensitive(True)

			cie = self.glade_pref_window.get_widget("custom_imager_entry")
			preferences.opt["open_image_custom"] = cie.get_text()

			cme = self.glade_pref_window.get_widget("custom_mailer_entry")
			preferences.opt["open_mail_custom"] = cme.get_text()

			cme = self.glade_pref_window.get_widget("custom_browser_entry")
			preferences.opt["open_browser_custom"] = cme.get_text()

		else :
			#cap.hide()
			cap.set_sensitive(False)

		preferences.opt["open_with"] = ["auto", "custom"][index]
		preferences.update_open_with()

	def imager_action(self, entry) :
		preferences.opt["open_image_custom"] = entry.get_text()
		preferences.update_open_with()

	def mailer_action(self, entry) :
		preferences.opt["open_mail_custom"] = entry.get_text()
		preferences.update_open_with()

	def browser_action(self, entry) :
		preferences.opt["open_browser_custom"] = entry.get_text()
		preferences.update_open_with()

	def email_methods_action(self, combobox) :
		index = combobox.get_active()

		st = self.glade_pref_window.get_widget("server_table")
		if index == 2 :
			#st.show()
			st.set_sensitive(True)

			se = self.glade_pref_window.get_widget("server_entry")
			se.set_text(preferences.opt["mail_server"])
		else :
			#st.hide()
			st.set_sensitive(False)

		preferences.opt["mail_method"] = self.mail_methods[index]

	def server_entry_action(self, entry) :
		preferences.opt["mail_server"] = entry.get_text()

	def promise_action(self, button) :
		self.glade_pref_window.get_widget("warning_frame").hide()

		self.opt_table_create()
		self.opt_table_fill()

		self.glade_pref_window.get_widget("options_filter").show()
		self.glade_pref_window.get_widget("options_table").show()

	def options_filter_action(self, entry) :
		self.opt_table_fill(entry.get_text())

	def options_table_action(self, cellrenderertext, index, text) :
		self.opts_liststore[index][1] = text
		key = self.opts_liststore[index][0]
		val = self.opts_liststore[index][1]
		if isinstance(preferences.opt[key], bool) :
			conv = config.string2bool
		elif isinstance(preferences.opt[key], list) :
			conv = partial(string.split, sep=" ")
		else :
			conv = type(preferences.opt[key])
		preferences.opt[key] = conv(val)
		print preferences.opt[key]

	def close_action(self, button, event=None) :
		preferences.save_user()
		self.pref_window.destroy()

	############
	def cat_create(self) :
		"""
		This function fills the list of the sections.
		"""
		cat_table = self.glade_pref_window.get_widget("cat_table")

		cat_liststore = gtk.ListStore(str, str)
		cat_table.set_model(cat_liststore)

		tvcolumn_icon = gtk.TreeViewColumn('icon')
		tvcolumn_cat  = gtk.TreeViewColumn('category')

		tvcolumn_icon.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		tvcolumn_cat.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		cat_table.append_column(tvcolumn_icon)
		cat_table.append_column(tvcolumn_cat)

		cell_icon = gtk.CellRendererPixbuf()
		cell_cat = gtk.CellRendererText()

		tvcolumn_icon.pack_start(cell_icon, True)
		tvcolumn_cat.pack_start(cell_cat, True)

		tvcolumn_icon.set_attributes(cell_icon, stock_id=0)
		tvcolumn_cat.set_attributes(cell_cat, text=1)

		cat_liststore.append([gtk.STOCK_EXECUTE, _("General")])
		cat_liststore.append([gtk.STOCK_COLOR_PICKER, _("Testing")])
		cat_liststore.append([gtk.STOCK_BOLD, _("Applications")])
		cat_liststore.append([gtk.STOCK_PREFERENCES, _("Options editor")])

		self.cat_table_action(cat_table, None)

	def opt_table_create(self) :
		"""
		This function shows the options editor.
		"""
		self.opts_liststore = gtk.ListStore(str, str)

		opts_widget = self.glade_pref_window.get_widget("options_table")
		opts_widget.set_model(self.opts_liststore)

		name_column =  gtk.TreeViewColumn('Name')
		name_column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		opts_widget.append_column(name_column)

		val_column =  gtk.TreeViewColumn('Value')
		val_column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		opts_widget.append_column(val_column)

		name_cell = gtk.CellRendererText()
		name_cell.set_property("editable", True)
		name_column.pack_start(name_cell)
		name_column.set_attributes(name_cell, text=0)

		val_cell = gtk.CellRendererText()
		val_cell.set_property("editable", True)
		val_column.pack_start(val_cell)
		val_column.set_attributes(val_cell, text=1)
		val_cell.connect("edited", self.options_table_action)

	def opt_table_fill(self, mask="") :
		"""
		This function fills the option editor fields.
		"""
		self.opts_liststore.clear()

		keys = preferences.opt.keys()
		keys.sort()

		for key in keys :
			val = preferences.opt[key]
			try :
				be = mask == "" or \
					re.search(mask,  key ) is not None or \
					re.search(mask, str(val)) is not None
			except :
				be = False
			if be :
				sval = ""
				if isinstance(val, str) :
					sval = val
				elif isinstance(val, list) :
					sval = string.join(val)
				else :
					sval = str(val)
				self.opts_liststore.append([key, sval])

