#!/usr/bin/python
# -*- coding: utf-8 -*-
##      report.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import encodings
import gettext
import gtk
import string
import textwrap
import time
from encodings.base64_codec import base64_encode
from functools import partial

import collect_funcs
import debug
import globs
import gui_funcs
import mailer
from pref import preferences

_ = gettext.gettext

class Report(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)
		self.width = 71
		self.head = ""
		self.body = ""
		self.sep0 = self.insert_between("", "-", self.width) + "\n"
		self.sep1 = self.insert_between(" 8< ", "-", self.width) + "\n"
		self.sep2 = "- " * (self.width / 2) + "-" * (self.width % 2) + "\n"
		self.user_data = {}

	def prepare(self, hosts) :
		self.number_of_tests = min(hosts,  key=lambda h : h.out if h.enable else 10000).out
		self.hosts = hosts

		zone0 = filter (lambda h : h.zone == 0 and h.enable, self.hosts)
		zone1 = filter (lambda h : h.zone == 1 and h.enable, self.hosts)
		zone2 = filter (lambda h : h.zone == 2 and h.enable, self.hosts)
		self.zones = [zone0, zone1, zone2]

		self.subj = "%s report %s" % (preferences.const["app_name"],
			time.strftime("%d/%m/%Y %H:%M:%S", preferences.const["begin_date"]))

		self.head = \
			_("To: maintenance crew of %s\n") % preferences.opt["prov_name"] +\
			_("Subject: %s\n") % self.subj +\
			_("Body: \n") +\
			self.sep1

		self.body_head = self.make_body_head()

		self.body_head += \
			self.sep0 +\
			self.make_short_report()[0] +\
			self.sep0

		self.body_tail = self.sep0 +\
			_("Results of the ping test:\n") +\
			self.make_tests_report() +\
			self.sep0 + "\n" +\
			self.sep0 +\
			_("What hosts and for what reason were tested:\n") +\
			self.make_explanation_report() +\
			self.sep0 + "\n"

		inc_os = preferences.opt["include_about_os"]
		inc_inter = preferences.opt["include_about_interfaces"]
		if inc_os or inc_inter :
			self.body_tail += _("Information about your system:\n\n")
		if inc_os :
			self.body_tail += collect_funcs.get_information_about_os()[1]
		if inc_inter :
			self.body_tail += collect_funcs.get_information_about_interfaces()[1]
		if preferences.opt["include_rubbish"] :
			self.body_tail += self.make_rubbish()

	def build(self) :
		"""
		This function creates the all report.
		"""
		self.body_user = self.make_user_report()
		res = self.body_head + self.body_user

		res = self.wrap(res)

		res += self.body_tail
		if preferences.opt["include_rubbish"] and \
				self.user_data["yes"] != "0" :
			body = self.encode(self.body_user,
					preferences.opt["encoding_userwish"])
			res += base64_encode(body)[0]
		else :
			res += "\n"

		return res

	def make_body_head(self) :
		"""
		This function forms the formated report about the test conditions.
		"""
		if preferences.opt["cable"] :
			cn = "LAN"
		else :
			cn = "MODEM"

		un = partial(unicode, encoding="utf-8")
		messages = map(un, [
				_("Good day!"),
				_("Subscriber name: "),
				_("Test made from: "),
				_("Packet size: "),
				_("Timeout (ms): "),
				_("Speed (gateway / center node): "),
				_("Begin of testing: "),
				_("End of testing: "),
		])
		maxlen = str(max(map(len, messages)))
		body_head = \
			("%-" + maxlen + "s") % messages[0] + "\n\n" +\
			("%-" + maxlen + "s") % messages[1] +\
			"%s\n" % preferences.opt["prov_subscriber_name"] +\
			("%-" + maxlen + "s") % messages[2] +\
			"%s [%s], %s, %s %s\n" % (preferences.opt["hostname"],
				preferences.opt["ip"],
				cn, preferences.const["app_name"],
				preferences.const["app_version"]
			) +\
			("%-" + maxlen + "s") % messages[3] +\
			"%d\n" % preferences.opt["ping_size"] +\
			("%-" + maxlen + "s") % messages[4] +\
			"%d\n" % preferences.opt["ping_timeout"] +\
			("%-" + maxlen + "s") % messages[5] +\
			("%s / %s\n\n" % \
				(collect_funcs.get_speed_discard(preferences.opt["speed_gateway"]),
				collect_funcs.get_speed_discard(preferences.var["center_host"]))
			) +\
			("%-" + maxlen + "s") % messages[6] +\
			"%s\n" % time.strftime("%d/%m/%Y %H:%M:%S", preferences.const["begin_date"])+\
			("%-" + maxlen + "s") % messages[7] +\
			"%s\n" % time.strftime("%d/%m/%Y %H:%M:%S", preferences.const["end_date"])

		wrap = partial(textwrap.wrap, width=self.width)
		join = partial(string.join, sep="\n")
		join1 = partial(string.join, sep=("\n%" + maxlen + "s") % "") # spaces before values
		split = partial(string.split, sep="\n")

		return str(join(map(join1, map(wrap, split(body_head)))))

	def make_rubbish(self) :
		"""
		This function forms machine readable information.
		"""
		rubbish = \
			"***\n" +\
			"version:%s\n" % preferences.opt["report_version"] +\
			"org:%s\n" % preferences.opt["prov_subscriber_name"] +\
			"from:%s\n" % preferences.opt["prov_subscriber_email"] +\
			"ipaddr:%s\n" % preferences.opt["ip"] +\
			"date:%s\n" % time.strftime("%Y%m%d%H%M%S", preferences.const["begin_date"]) +\
			"subject:%s\n" % self.subj

		for zone in [0,1,2] :
			zone_str = ",".join(map(lambda h : h.ip, self.zones[zone]))
			rubbish += "zone%d:%s\n" % (zone+1, zone_str)

		hosts_by_user = filter(lambda h : h.enable and h.flag == "User", self.hosts)
		rubbish += \
			"traced:%s\n" % string.join(map(lambda h: h.ip, hosts_by_user), ",") +\
			"remark:\n"
		return rubbish

	def make_short_report(self) :
		"""
		This function forms a recommendation and a short report for each zone of
		responsibility.
		"""
		(ltest, lpsize, lout) = self.get_test_criterion()
		test =  ["unacceptable", "poor", "quite enough", "careful"]
		psize = [
			"very small", "small (satisfactorily)",
			"medium (good!)", "big (excellent!)"]
		out = psize

		test_report = ""
		test_report += _("     You made %s test: " % test[ltest]) +\
			_("%s size of packages was used " % psize[lpsize]) +\
			_("(%d bytes), ") % preferences.opt["ping_size"] +\
			_("number of pings (%s) is %s." % ("%d", out[lout])) \
				% self.number_of_tests
		test_report += "\n"

		zoneq = [
				"serious problems", "minor problem",
				"good rate of communication",
				"perfect rate of communication"]
		lost = ["Too big", "Wholly satisfactory", "Small (good!)",
				"Tiny (excellent!)"]
		ms = lost
		zoneq0 = -1
		for fhosts in self.zones :
			if fhosts == [] :
				continue
			(lzoneq, llost, lms) = self.get_communication_criterion(fhosts)
			z = fhosts[0].zone
			zone_desc = (
				preferences.const["prov_zones"][z][0] +
				preferences.const["prov_zones"][z][1])\
					% preferences.opt["prov_name"]

			if preferences.var["prov_enable_zones"] :
				test_report += \
					_("     Check have detected %s %s. " % (zoneq[lzoneq], "%s")) \
						% zone_desc
			else :
				test_report += \
					_("     Check have detected %s. " % zoneq[lzoneq])

			test_report += \
				_("%s part of packages lost (%s..%s%%%%). " \
					% (lost[llost[0]], "%.1f", "%.1f")) \
					% (llost[1], llost[2]) +\
				_("%s latency (%s..%s ms). " % (ms[lms[0]], "%.1f", "%.1f")) \
					% (lms[1], lms[2])
			test_report += "\n"
			if fhosts[0].zone == 0 :
				zoneq0 = lzoneq
		total = [
			_("It is necessary to sent this report!\n"),
			_("There is no much reason to sent this report.\n"),
			_("There is no apparent reason to sent this report.\n")
		]
		ltotal = self.get_total_criterion(ltest, zoneq0)
		test_report = total[ltotal] + test_report
		return (test_report, ltotal)

	def get_test_criterion(self) :
		"""
		This function calculates appraisal for the test conditions.
		"""
		lpsize = collect_funcs.estimate(
					0 - preferences.opt["ping_size"],
						preferences.opt["ping_psize_excel"],
						preferences.opt["ping_psize_good"],
						preferences.opt["ping_psize_not_so_bad"]
						)
		lout = collect_funcs.estimate(
					0 - self.number_of_tests,
						preferences.opt["ping_out_excel"],
						preferences.opt["ping_out_good"],
						preferences.opt["ping_out_not_so_bad"]
						)
		ltest = min(lpsize, lout)
		return (ltest, lpsize, lout)

	def get_communication_criterion (self, hosts):
		"""
		This function calculates appraisal for the connection quality.
		"""
		minlat = 10000
		maxlat = -1
		minloss = 10000
		maxloss = -1
		for host in hosts :
			latavg = host.cmsavg()
			lossper = host.clossper()
			minlat = min(minlat, latavg)
			maxlat = max(maxlat, latavg)
			minloss = min(minloss, lossper)
			maxloss = max(maxloss, lossper)
		ll = collect_funcs.estimate(
			maxloss, preferences.opt["ping_loss_excel"],
			preferences.opt["ping_loss_good"],
			preferences.opt["ping_loss_not_so_bad"])
		lm = collect_funcs.estimate(
			maxlat, preferences.opt["ping_lat_excel"],
			preferences.opt["ping_lat_good"],
			preferences.opt["ping_lat_not_so_bad"])
		lzoneq = min(ll, lm)
		return (
			lzoneq,
			[ll, minloss, maxloss],
			[lm, minlat,  maxlat ])

	def get_total_criterion(self, ltest, zoneq0) :
		"""
		This function calculates recommendation.
		"""
		RD = 0
		YL = 1
		GR = 2
		tag = [
				[YL, GR, GR, GR],
				[YL, YL, GR, GR],
				[RD, YL, GR, GR],
				[RD, YL, GR, GR],
		]
		return tag[ltest][zoneq0]

	def make_tests_report(self) :
		"""
		This function forms the table with the test statistic.
		"""
		first_col_len = 22
		res = self.sep2 +\
			str(("%-"+`first_col_len`+"s :%s") %\
			(unicode(_("Host's name"), "utf-8"),
			"   out  loss : loss% : ms min : ms max : ms avg\n"
			))+\
			self.sep2
		for fhosts in self.zones :
			if fhosts == [] :
				continue
			strg = preferences.const["prov_zones"][fhosts[0].zone][0]
			try :
				zone_desc = strg % preferences.opt["prov_name"]
			except :
				zone_desc = strg
			res += self.insert_between(zone_desc, "=", self.width) + "\n"
			for host in fhosts :
				hname = host.hostname
				if len(hname) > first_col_len :
					hname = hname[:(first_col_len-3)] + "..."
				res += \
					("%-"+`first_col_len`+"s :") % hname +\
					"%6d "	  % host.out +\
					"%6d :"	 % host.loss +\
					"%5.1f%% :" % host.clossper() +\
					"%6.1f :"   % host.msmin +\
					"%6.1f :"   % host.msmax +\
					"%6.1f\n"   % host.cmsavg()
		return res

	def make_explanation_report(self) :
		"""
		This function forms a explanation why several hosts was tested.
		"""
		routes = ""
		res = self.sep2 +\
			str("%30s : %-38s\n" % \
			(unicode(_("Host's name"), "utf-8"),
			unicode(_("Explanation"), "utf-8"))) + \
			self.sep2
		for fhosts in self.zones :
			if fhosts == [] :
				continue
			strg = preferences.const["prov_zones"][fhosts[0].zone][0]
			try :
				zone_desc = strg % preferences.opt["prov_name"]
			except :
				zone_desc = strg
			res += self.insert_between(zone_desc, "=", self.width) + "\n"
			for host in fhosts :
				res += "%30s" % host.hostname + \
				" : [" + host.ip + "]  " + host.flag + "\n"
				if host.flag == "User" :
					routes += self.sep2 + host.traceout
		if routes != "" :
			routes = "\n" + self.sep0 +\
				_("I has a hunch bad communication with a hosts.  There are results of tracing:\n")+\
				routes
		return res + routes

	def insert_between(self, strg, char, length):
		strg_len = len(unicode(strg, "utf-8"))
		if strg_len >= length : return strg
		part_len = (length - strg_len) / 2
		part = char * part_len
		tail = char * ((length - strg_len) % 2)
		return part + strg + part + tail

	def make_user_report(self) :
		"""
		This function formats wish of the user.
		"""
		yn_ind = self.user_data["yes"]
		y = _("No")
		[name, phone, address, date, time] = ["", "", "", "", ""]
		if yn_ind != "0" :
			if yn_ind == "1" : y = _("Yes")
			else:			  y = _("Don't known")
			name	= self.user_data["name"]
			phone   = self.user_data["phone_number"]
			address = self.user_data["address"]
			date	= self.user_data["date"]
			if self.user_data["time_type"] == "2" :
				time = _("Between %02d:%02d and %02d:%02d") %\
					(int(self.user_data["time_h1"]),
					int(self.user_data["time_m1"]),
					int(self.user_data["time_h2"]),
					int(self.user_data["time_m2"]),
					)
			elif self.user_data["time_type"] == "1" :
				time = _("Before %02d:%02d") %\
						(int(self.user_data["time_h1"]),
						int(self.user_data["time_m1"]),
						)
			elif self.user_data["time_type"] == "0" :
				time = _("After %02d:%02d") %\
						(int(self.user_data["time_h1"]),
						int(self.user_data["time_m1"]),
						)
		res = \
			_("Do you need visit of the maintenance crew (Yes, No, Don't know):\n%s\n") % y +\
			_("If you need:\n") +\
			_("Name and surname of contact person:\n%s\n") % name+\
			_("telephone number:\n%s\n") % phone +\
			_("Address for the maintenance crew:\n%s\n") % address+\
			_("At your convenience? (Our schedule: %s):\n") % preferences.opt["prov_schedule"] +\
			_("convenient date:\n%s\n") % date +\
			_("convenient time (for example after 14-00):\n%s\n") % time

		if globs.speedtestnetgui.add_to_report and globs.speedtestnetgui.date is not None:
			res += str("\n%s\n" % globs.speedtestnetgui.get_text_for_report())

		return res

	def save(self, filename, window=None) :
		"""
		This function saves report to a file.
		"""
		f = None
		try :
			report = self.encode(self.head + self.build(),
							preferences.opt["encoding_report_file"])
			f = open(filename, "w")
			f.write(report)
		except Exception, e:
			gui_funcs.show_message(window, gtk.MESSAGE_ERROR,
					_("Error!"), _("Can't write %(file)s : %(reason)s")\
							% {"file" : filename, "reason" : str(e)})
		finally :
			if f: f.close()

	def send(self, window=None) :
		"""
		This function sends the report mail.
		"""
		if preferences.opt["prov_maintenance_crew_email"] == "" :
			gui_funcs.show_message(window, gtk.MESSAGE_ERROR,
				_("Error!"),
				_("Can't find email address of the maintenance crew."))
			return
		m = mailer.Mailer()
		m.subj = self.subj
		m.body = self.encode(self.build(),
						preferences.opt["encoding_report_mail"])
		try :
			m.send()
		except Exception, e :
			gui_funcs.show_message(window, gtk.MESSAGE_ERROR,
				_("Error!"),
				_("Can't send email: ") + str(e))

	def wrap(self, strg) :
		wrap = partial(textwrap.wrap, width=self.width)
		join = partial(string.join, sep="\n")
		split = partial(string.split, sep="\n")
		return str(join(map(join, map(wrap, split(unicode(strg, "utf-8"))))))

	def encode(self, text, enc) :
		if enc != "":
			codec = encodings.search_function(enc)
			if codec is None :
				print "Bad encoding name: " + enc
			else :
				text = unicode(text, "utf-8")
				text = codec.encode(text)[0]
		return text

