#!/usr/bin/python
# -*- coding: utf-8 -*-
##      speedgui.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk
import threading

import collect_funcs
import debug
import gui_funcs
from pref import preferences

_ = gettext.gettext

proto_discard = "discard"
proto_iperf   = "iperf"

class SpeedGui(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)

	def run(self, window, host="") :
		"""
		This function shows window of data rate testing.
		"""
		self.glade_speedwin = gtk.glade.XML(preferences.opt["glade_speed"])

		self.speed_window = self.glade_speedwin.get_widget("speed_window")
		self.speed_window.set_transient_for(window)

		if preferences.opt["speed_proto"] == proto_discard :
			self.glade_speedwin.get_widget("speed_discard").set_active(True)
		else :
			self.glade_speedwin.get_widget("speed_iperf").set_active(True)

		self.glade_speedwin.signal_connect("on_speed_discard_clicked",
								lambda b : self.set_proto(b, proto_discard))
		self.glade_speedwin.signal_connect("on_speed_iperf_clicked",
								lambda b : self.set_proto(b, proto_iperf))

		self.glade_speedwin.signal_connect("on_close_clicked", self.on_close)
		self.glade_speedwin.signal_connect("on_run_clicked", self.on_run)

		if host != "" :
			self.glade_speedwin.get_widget("server").set_text(host)
			self.on_run(None)

	def on_run(self, button) :
		th = threading.Thread(target=self._on_run, name="getting speed")
		th.setDaemon(True)
		th.start()

	def _on_run(self) :
		server = self.glade_speedwin.get_widget("server").get_text()
		label = self.glade_speedwin.get_widget("speed_label")

		gui_funcs.gtk_threads_enter()
		label.set_text(_("Measuring..."))
		gui_funcs.gtk_iteration()
		gui_funcs.gtk_threads_leave()

		if preferences.opt["speed_proto"] == proto_discard :
			speed = collect_funcs.get_speed_discard(server)
		else :
			speed = collect_funcs.get_speed_iperf(server)

		gui_funcs.gtk_threads_enter()
		try : # window may be closed
			label.set_text(speed)
		except :
			pass
		gui_funcs.gtk_threads_leave()

	def on_close(self, button) :
		self.speed_window.destroy()

	def set_proto(self, radioButton, proto) :
		preferences.opt["speed_proto"] = proto

