#!/usr/bin/python
# -*- coding: utf-8 -*-
##      speedtestnetgui.py
##
##
## Copyright (C) 2009 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk
import os
import threading
import time

import collect_funcs
import debug
import gui_funcs
import speedtestnet
from pref import preferences

_ = gettext.gettext

class SpeedTestNetGui(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)

		self.init()

	def init(self) :
		"""
		This function initializes the variables for the report
		"""
		self.add_to_report = False
		self.date = None
		self.server = ""
		self.dist = 0
		self.latency = 0
		self.down_speed = 0
		self.up_speed = 0

	def run(self) :
		"""
		This functions opens the interface for speedtest.net
		"""
		self.init()

		self.glade_window = gtk.glade.XML(preferences.opt["glade_speedtestnet"])

		# main dialog
		self.window = self.glade_window.get_widget("window")
		#self.window.set_transient_for(window)

		# progressbar
		self.progress = self.glade_window.get_widget("progressbar")
		self.progress.set_fraction(0)

		# servers combobox
		self.servers_list_combobox = self.glade_window.get_widget("servers_list")
		self.servers_model = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
		self.servers_list_combobox.set_model(self.servers_model)

		renderer_image = gtk.CellRendererPixbuf()
		self.servers_list_combobox.pack_start(renderer_image, False)
		self.servers_list_combobox.set_attributes(renderer_image, pixbuf = 0)

		renderer_text = gtk.CellRendererText()
		self.servers_list_combobox.pack_start(renderer_text, True)
		self.servers_list_combobox.set_attributes(renderer_text, text = 1)

		renderer_dist = gtk.CellRendererText()
		self.servers_list_combobox.pack_start(renderer_dist, True)
		self.servers_list_combobox.set_attributes(renderer_dist, text = 2)

		# signals
		self.glade_window.signal_connect("on_toreport_clicked", self.on_toreport)
		self.glade_window.signal_connect("on_close_clicked", self.on_close)
		self.glade_window.signal_connect("on_run_clicked", self.on_run)

		# speedtest.net client
		self.speedtn = speedtestnet.SpeedTestNet()

		# start setting up
		self.set_progress_label(_("Setting up..."))
		th = threading.Thread(target=self.config, name="speedtestnet_config")
		th.setDaemon(True)
		th.start()

	def on_toreport(self, button) :
		"""
		'Add the results to the report' checkbutton handler
		"""
		self.add_to_report = button.get_active()

	def on_close(self, button) :
		"""
		Close button handler
		"""
		self.window.destroy()

	def on_run(self, button) :
		"""
		Run button handler
		"""
		self.glade_window.get_widget("run").set_sensitive(False)

		self.update_speed_label(_("n/a"), "latency")
		self.update_speed_label(_("n/a"), "down_speed")
		self.update_speed_label(_("n/a"), "up_speed")

		th = threading.Thread(target=self._on_run, name="speedtestnet_test")
		th.setDaemon(True)
		th.start()

	def _on_run(self) :
		"""
		This function makes speedtest.net tests
		"""
		server = self.servers_list_combobox.get_active()

		self.server = self.speedtn.servers[server]["name"]
		self.dist = int(self.speedtn.servers[server]["dist"])

		self.update_speed_label(_("Loading..."), "latency")
		self.set_progress_label(_("Latency test..."))
		self.latency = self.get_latency(server)

		self.set_progress_label(_("Download test..."))
		self.update_speed_label(_("Loading..."), "down_speed")
		self.down_speed = self.get_down_speed(self.latency, server)

		self.set_progress_label(_("Upload test..."))
		self.update_speed_label(_("Loading..."), "up_speed")
		self.up_speed = self.get_up_speed(self.latency, server)

		self.set_progress_label(_("Completed"))
		gui_funcs.gtk_threads_enter()
		self.glade_window.get_widget("run").set_sensitive(True)
		gui_funcs.gtk_threads_leave()

		self.date = time.localtime()

	def config(self) :
		"""
		This function gets and process the configuration file.
		"""
		n = 5
		step = 1.0 / n

		gui_funcs.gtk_threads_enter()
		self.window.set_sensitive(False)
		gui_funcs.gtk_threads_leave()

		self.set_progress(0)
		self.speedtn.get_config()

		self.set_progress(step)
		self.speedtn.parse_config()

		self.set_progress(2 * step)
		self.speedtn.servers_sort_by_dist()

		self.set_progress(3 * step)
		self.fill_servers_combobox()

		self.set_progress(4 * step)
		self.fill_user_data()

		self.set_progress(1)

		self.set_progress_label(_("Completed"))

		gui_funcs.gtk_threads_enter()
		self.window.set_sensitive(True)
		gui_funcs.gtk_threads_leave()

	def fill_servers_combobox(self) :
		"""
		This function fills list of the speedtest.net servers (ComboBox
		widget).
		"""
		for server in self.speedtn.servers :
			fname = preferences.opt["earth_flags"] + "/%s.png" % (server["countrycode"].lower())
			if os.path.exists(fname) :
				image = gtk.gdk.pixbuf_new_from_file(fname)
			else :
				image = None

			entry = "%s" % (
						server["name"],
						#server["country"], #server["sponsor"],
						#float(server["lat"]), float(server["lon"]),
					)
			dist = _("≅ %d km") % int(server["dist"])

			gui_funcs.gtk_threads_enter()
			self.servers_model.append([image, entry, dist])
			gui_funcs.gtk_threads_leave()

		self.servers_list_combobox.set_active(0)

	def fill_user_data(self) :
		"""
		This function fills fields about user's host or gateway.
		"""
		self.glade_window.get_widget("user_ip").set_text(self.speedtn.client["ip"])

		lat = float(self.speedtn.client["lat"])
		latM = (lat - int(lat)) * 60
		latS = (latM - int(latM)) * 60
		lon = float(self.speedtn.client["lon"])
		lonM = (lon - int(lon)) * 60
		lonS = (lonM - int(lonM)) * 60

		latN = "%d°%02d'%02d\"" % (int(lat), int(latM), int(latS))
		lonW = "%d°%02d'%02d\"" % (int(lon), int(lonM), int(lonS))

		gui_funcs.gtk_threads_enter()
		self.glade_window.get_widget("user_coordinates").set_text(
				_("%sN, %sW") % (latN, lonW)
		)
		self.glade_window.get_widget("user_isp").set_text(self.speedtn.client["isp"])
		gui_funcs.gtk_threads_leave()

	def get_latency(self, server) :
		"""
		This function makes latency test.
		"""
		n = preferences.opt["speedtest_latency_attempts"]
		step = float(1) / n

		self.set_progress(0)

		latency = 0
		for i in xrange(n) :
			self.set_progress(i * step)
			latency += self.speedtn.latency_test(server)

		latency /= float(n)

		self.glade_window.get_widget("latency").set_text(_("%d ms") % (latency*1000))

		self.set_progress(1)

		return latency

	def get_down_speed(self, latency, server) :
		"""
		This function makes download test.
		"""
		n = preferences.opt["speedtest_down_attempts"]
		step = float(1) / n

		self.set_progress(0)

		speed = 0
		for i in xrange(n) :
			self.set_progress(i * step)
			(bytes_, seconds) = self.speedtn.download_test(server)
			speed += bytes_ / (seconds - latency)

			self.update_speed_label(speed/(i+1), "down_speed")

		speed /= float(n)
		self.update_speed_label(speed, "down_speed")

		self.set_progress(1)

		return speed

	def get_up_speed(self, latency, server) :
		"""
		This function makes upload test.
		"""
		n = preferences.opt["speedtest_up_attempts"]
		step = float(1) / n

		self.set_progress(0)

		speed = 0
		for i in xrange(n) :
			self.set_progress(i * step)
			(bytes_, seconds) = self.speedtn.upload_test(server)
			speed += bytes_ / (seconds - latency)

			self.update_speed_label(speed/(i+1), "up_speed")

		speed /= float(n)
		self.update_speed_label(speed, "up_speed")

		self.set_progress(1)

		return speed

	def update_speed_label(self, label, name) :
		"""
		This function updates upload/download speed labels.
		"""
		if isinstance(label, float)  or isinstance(label, int) :
			l = "%s\n(%s)" %\
					(_("%s/s") % collect_funcs.bytes_fmt(label),
					_("%s/s") % collect_funcs.bits_fmt(label*8))
		else :
			l = label

		gui_funcs.gtk_threads_enter()
		self.glade_window.get_widget(name).set_text(l)
		gui_funcs.gtk_threads_leave()

	def set_progress(self, fraction) :
		gui_funcs.gtk_threads_enter()
		self.progress.set_fraction(fraction)
		gui_funcs.gtk_threads_leave()

	def set_progress_label(self, label) :
		gui_funcs.gtk_threads_enter()
		self.progress.set_text(label)
		gui_funcs.gtk_threads_leave()


	def get_text_for_report(self) :
		"""
		This function builds shot report about the data rate test.
		"""
		return _(\
			"I have performed a data rate test using the speedtest.net " +\
			"services.  Test was completed (%(date)s) with a server in " +\
			"%(server)s (about %(km)d km from me).  I got %(lat)d ms of " +\
			"the latency, %(down)s of the download speed " + "and %(up)s of " +\
			"the upload speed.") % {
					"date": time.strftime("%d/%m/%Y %H:%M:%S", self.date),
					"server" : self.server,
					"km" : self.dist,
					"lat" : self.latency*1000,
					"down" : "%s (%s)" % (
						(_("%s/s") % collect_funcs.bytes_fmt(self.down_speed),
						_("%s/s") % collect_funcs.bits_fmt(self.down_speed*8))
						),
					"up" : "%s (%s)" % (
						(_("%s/s") % collect_funcs.bytes_fmt(self.up_speed),
						_("%s/s") % collect_funcs.bits_fmt(self.up_speed*8))
						),
				}

if __name__ == "__main__":
	gtk.gdk.threads_init()
	stng = SpeedTestNetGui()
	stng.run()
	gtk.main()

