#!/usr/bin/python
# -*- coding: utf-8 -*-
##      tester.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gobject
import gtk
import gtk.glade
import math
import os
import re
import threading
import time

import askwindow
import debug
import globs
import googlemap
import gui_funcs
import history_graph
import netmap
import nmap
import prefwindow
import restore
import speedgui
from pref import preferences

try :
	import sexy
	preferences.var["sexy_exist"] = True
except ImportError :
	preferences.var["sexy_exist"] = False

_ = gettext.gettext

class TesterGui(debug.Debug) :
	def __init__(self):
		debug.Debug.__init__(self)
		self.redraw_table = lambda hosts : (
				globs.worktable.redraw_table_widget(hosts),
				self.set_show_host_widgets(False)
		)
		self.run_main_window()
		self.init_popup_menu()

		self.reconfigure(None)

		self.current_file = ""

		self.check_first_start()

		self.host_field.grab_focus()

	def run_main_window (self) :
		"""
		This function shows main window.
		"""
		# to show main window
		self.glade_main_window = gtk.glade.XML(preferences.opt["glade_main"])

		# to create gtk.Entry
		if preferences.var["sexy_exist"] :
			self.host_field = sexy.IconEntry()
			self.host_field.add_clear_button()
		else :
			self.host_field = gtk.Entry()
		pfhn = self.glade_main_window.get_widget("place_for_host_name")
		pfhn.add(self.host_field)

		# to create progressbar in the stastusbar
		self.testing_progress = gtk.ProgressBar()
		self.testing_progress.hide()
		stbar = self.glade_main_window.get_widget("statusbar")
		stbar.pack_end(self.testing_progress, expand=False)

		self.set_show_test_widgets("play")
		self.set_show_host_widgets(False)

		# to set speedtest.net and google icons
		sti = self.glade_main_window.get_widget("speedtest_image")
		sti.set_from_file(preferences.opt["speedtest_icon"])
		gi = self.glade_main_window.get_widget("google_image")
		gi.set_from_file(preferences.opt["google_icon"])

		# connect to signals
		self.main_window = self.glade_main_window.get_widget("main_window")
		self.main_window.connect("delete-event", self.main_on_close)
		self.main_window.connect("drag-data-received", self.drag_data_received)

		self.main_window.drag_dest_set(gtk.DEST_DEFAULT_ALL,
			[
				('text/plain',0, 0),
				('STRING', 0, 2),
				('UTF8_STRING', 0, 4),
			],
			gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE
		)

		self.set_title("")
		preferences.var["font"] = self.main_window.get_style().font_desc.get_family()

		compl = gtk.EntryCompletion()
		self.host_name_history_liststore = gtk.ListStore(gobject.TYPE_STRING)
		self.host_name_history_liststore_init()
		compl.set_model(self.host_name_history_liststore)
		compl.set_text_column(0)

		self.host_field.set_completion(compl)
		self.host_field.show_all()

		self.glade_main_window.signal_connect("on_add_host_clicked", self.on_add_host)
		self.glade_main_window.signal_connect("on_play_clicked", self.on_play_clicked)
		self.glade_main_window.signal_connect("on_quickplay_clicked", self.on_quickplay_clicked)
		self.glade_main_window.signal_connect("on_stop_clicked", self.on_stop_clicked)
		self.glade_main_window.signal_connect("on_pause_clicked", self.on_pause_clicked)
		self.glade_main_window.signal_connect("on_clear_counters", self.clear_counters)
		self.glade_main_window.signal_connect("on_clear_list", self.on_clear_hosts)
		self.glade_main_window.signal_connect("on_work_area_button_press_event", self.on_work_area_click)
		self.glade_main_window.signal_connect("on_send_report", self.send_report)

		self.glade_main_window.signal_connect("on_menu_openlist", self.open_hosts)
		self.glade_main_window.signal_connect("on_menu_savelist", self.save_hosts)
		self.glade_main_window.signal_connect("on_menu_savelistas", self.save_hosts_as)
		self.glade_main_window.signal_connect("on_menu_reconfigure", self.reconfigure)
		self.glade_main_window.signal_connect("on_menu_quit", self.main_on_close)
		self.glade_main_window.signal_connect("on_menu_pref", self.show_preferences)
		self.glade_main_window.signal_connect("on_menu_enabledisable_host", self.toggle_selected)
		self.glade_main_window.signal_connect("on_menu_delete", self.delete_selected)
		self.glade_main_window.signal_connect("on_menu_reset_host_counters", self.reset_selected)
		self.glade_main_window.signal_connect("on_menu_show_history", self.show_history)
		self.glade_main_window.signal_connect("on_menu_get_host_speed", self.getspeed_slected)
		self.glade_main_window.signal_connect("on_menu_show_map", self.show_map)
		self.glade_main_window.signal_connect("on_menu_show_googlemap", self.show_googlemap)
		self.glade_main_window.signal_connect("on_menu_show_speed", self.show_speed)
		self.glade_main_window.signal_connect("on_menu_speetest", self.show_speedtest)
		self.glade_main_window.signal_connect("on_menu_about", self.on_menu_about)

		globs.testing.tests_planed = 0

	#####
	##### signals
	#####
	def drag_data_received(self, widget, drag_context, x, y, selection_data, info, timestamp) :
		"""
		Drag'n'drop handler
		"""
		th = threading.Thread(
			target=self._on_add_host,
			args=(selection_data.get_text(),))
		th.setDaemon(True)
		th.start()

	def on_add_host (self, button) :
		"""
		Add host button handler.
		"""
		th = threading.Thread(
			target=self._on_add_host,
			args=(self.host_field.get_text(),))
		th.setDaemon(True)
		th.start()

	def _on_add_host (self, host_name) :
		if host_name == "" :
			return

		gui_funcs.gtk_threads_enter()
		self.main_window.set_sensitive(False)
		gui_funcs.gtk_threads_leave()

		if self.adding_gui(host_name) :
			gui_funcs.gtk_threads_enter()
			self.host_name_history_liststore_add(host_name)
			self.host_field.set_text("")
			gui_funcs.gtk_threads_leave()

		gui_funcs.gtk_threads_enter()
		self.main_window.set_sensitive(True)
		self.host_field.grab_focus()
		gui_funcs.gtk_threads_leave()

	def on_play_clicked (self, button) :
		self.play(preferences.opt["ping_number_of_tests"])

	def on_quickplay_clicked (self, button) :
		self.play(1)

	def play(self, n) :
		"""
		This function starts the ping-test in a new thread.
		"""
		if globs.testing.hosts == [] or globs.testing.pings_run :
			return

		self.report_to_statusbar(_("Ping test started..."), 0)

		globs.testing.pings_run = True
		self.set_show_test_widgets("stop")

		th = threading.Thread(
			target=globs.testing.pings,
			args=(self.on_test_progress, self.on_test_end, n))
		th.setDaemon(True)
		th.start()

	def on_test_progress(self) :
		"""
		This function updates the interface during the test.
		"""
		globs.worktable.update_table(globs.testing.hosts)

		(hsum, hlen, hq) = reduce(
				lambda (s,l,q), h :
					(s+h.out, l+1, min(q, h.get_quality())) if h.enable else (s,l,q),
				globs.testing.hosts, (0, 0, 100))

		n = hlen * globs.testing.tests_planed
		if n == 0 :
			fr = 1.0
		else :
			fr = float(hsum) / n

		gui_funcs.gtk_threads_enter()

		self.testing_progress.set_fraction(fr)
		self.testing_progress.set_text("%.1f %%" % (fr*100))

		if hq < 100 :
			# progress in the title?
			self.update_trayicon(hq)

		gui_funcs.gtk_threads_leave()

	def on_test_end(self) :
		"""
		This function is called when the test is stopped.
		"""
		globs.worktable.update_table(globs.testing.hosts)
		self.set_show_test_widgets("play")
		self.clear_statusbar(0)

	def on_stop_clicked (self, button) :
		"""
		Stop button handler.
		"""
		if globs.testing.pings_run :
			self.on_pause_clicked(None)
			time.sleep(int(math.ceil(float(preferences.opt["ping_timeout"]) / 1000)))
			self.clear_counters(None)

	def on_pause_clicked (self, button) :
		"""
		Pause button handler.
		"""
		if globs.testing.pings_run :
			globs.testing.pings_run = False
			self.set_show_test_widgets("play")
			globs.testing.tests_planed -= preferences.opt["ping_number_of_tests"]

	def on_work_area_click (self, tree, event=None) :
		"""
		The "button-press-event" signal is emitted when a mouse button is
		pressed.
		Returns : True to stop other handlers from being invoked for the event.
		False to propagate the event further
		"""
		if event is None : return False

		if event.type == gtk.gdk.BUTTON_PRESS :
			(x, y) = map(int, event.get_coords())
			# Figure out which item they right clicked on
			path = tree.get_path_at_pos(x, y)
			# Get the selection
			sel = tree.get_selection()

			in_list = path is not None
			if event.button == 3 and in_list : # right mouse button on a host row
				# If they didnt right click on a currently selected row, change
				# the selection.
				if not sel.path_is_selected(path[0]) :
					sel.unselect_all()
					sel.select_path(path[0])

				# to show the popup menu
				self.ppmenu.popup(None, None, None, event.button, event.time)
				self.set_show_host_widgets(True)
				return True
			elif event.button == 1 or event.button == 2:
				if in_list : # left or middle mouse button on a host row
					self.set_show_host_widgets(True)
				else : # left or middle mouse button on emptiness
					# to delete any selection
					sel.unselect_all()
					self.set_show_host_widgets(False)
			return False

	def open_hosts(self, menu_item) :
		"""
		Open a hosts list menu item handler.
		"""
		th = threading.Thread(target=self._open_hosts,
						name="list opening")
		th.setDaemon(True)
		th.start()

	def _open_hosts(self) :
		r = restore.RestoreHosts()
		filename = r.open_list(self.main_window)

		if filename != "" :
			self.redraw_table(globs.testing.hosts)
			self.current_file = filename
			self.set_title(self.current_file)

	def save_hosts(self, menu_item) :
		"""
		Save the hosts list menu item handler.
		"""
		if self.current_file == "" :
			self.save_hosts_as(menu_item)
		else :
			r = restore.RestoreHosts()
			filename = r.save_list(globs.testing.hosts, self.current_file)
			if filename != "" :
				self.current_file = filename
				self.set_title(self.current_file)

	def save_hosts_as(self, menu_item) :
		"""
		Save as the hosts list menu item handler.
		"""
		r = restore.RestoreHosts()
		filename = r.save_list(globs.testing.hosts, window=self.main_window)
		if filename != "" :
			self.current_file = filename
			self.set_title(self.current_file)

	def reconfigure(self, menu_item) :
		"""
		Automatic configuration menu item handler.
		"""
		globs.testing.clear_hosts()
		th = threading.Thread(target=self.configure_hosts,
						name="configuration")
		th.setDaemon(True)
		th.start()

	def send_report (self, menu_item) :
		"""
		Prepare report button handler.
		"""
		if globs.testing.hosts == [] :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_ERROR,
				_("Error!"),
				_("Can't create the report. List of hosts is empty."))

			return

		aw = askwindow.AskWindow(globs.testing.hosts)
		aw.run(self.main_window)

	def main_on_close (self, window, event=None) :
		"""
		Quit handler.
		"""
		self.on_stop_clicked(None)

		# save user data
		preferences.save_user()
		self.host_name_history_liststore_save()

		# remove temporary files
		for filename in preferences.var["files_to_remove"] :
			try :
				os.remove(filename)
			except OSError :
				pass

		gtk.main_quit()

	def toggle_selected(self, menu_item, hosts=None):
		"""
		Enable/Disable handler.
		"""
		if hosts is None :
			hosts = globs.worktable.get_selected()

		for h in hosts :
			globs.testing.toogle_enable_host(h)

		self.redraw_table(globs.testing.hosts)

		self.update_trayicon()

	def delete_selected(self, menu_item):
		"""
		Delete handler.
		"""
		deleted = globs.worktable.get_selected()
		globs.testing.deleted_hosts += [h for h in globs.testing.hosts if h in deleted ]
		globs.testing.hosts = [h for h in globs.testing.hosts if h not in deleted ]
		self.redraw_table(globs.testing.hosts)

		self.update_trayicon()

	def reset_selected(self, menu_item):
		"""
		Reset statistics handler.
		"""
		reseted = globs.worktable.get_selected()
		globs.testing.hosts = map(lambda h : h.defaults() if h in reseted else h, globs.testing.hosts)
		self.redraw_table(globs.testing.hosts)

		self.update_trayicon()

	def getspeed_slected(self, menu_item) :
		"""
		Get speed (iperf, discard) handler.
		"""
		selected = globs.worktable.get_selected()[0]
		speed = speedgui.SpeedGui()
		speed.run(self.main_window, selected.hostname)

	def clear_counters(self, menu_item) :
		"""
		Reset all statistics handler.
		"""
		map(lambda h : h.defaults(), globs.testing.hosts)
		self.redraw_table(globs.testing.hosts)
		globs.testing.tests_planed = 0

		self.update_trayicon(-1)

	def on_clear_hosts(self, menu_item) :
		"""
		Clear table handler.
		"""
		# remove all hosts from the table
		globs.testing.clear_hosts()
		self.redraw_table(globs.testing.hosts)

		self.update_trayicon(-1)

	def show_preferences(self, menu_item) :
		"""
		Preferences menu item handler.
		"""
		p = prefwindow.PrefWindow()
		p.run(self.main_window)

	def show_map(self, menu_item) :
		"""
		Schematic map menu item handler.
		"""
		if not preferences.var["pydot_exist"] :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_WARNING,
				_("Error!"), _("You should to install pydot for using this feature, please."))
			return

		m = netmap.Map(globs.testing.hosts + globs.testing.deleted_hosts)
		m.run()

	def show_googlemap(self, menu_item) :
		"""
		Google maps menu item handler.
		"""
		m = googlemap.GMap(globs.testing.hosts + globs.testing.deleted_hosts)
		if not m.run() :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_WARNING,
				_("Error!"), _("No hosts to display on the Google map."))

	def show_speed(self, menu_item) :
		"""
		Speed to a host menu item handler.
		"""
		speed = speedgui.SpeedGui()
		speed.run(self.main_window)

	def show_speedtest(self, menu_item) :
		"""
		speedtest.net menu item handler.
		"""
		globs.speedtestnetgui.run()

	def show_history(self, menu_item) :
		"""
		History of the host testing menu item handler.
		"""
		if not preferences.var["gnuplot_exist"] :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_WARNING,
				_("Error!"), _("You should to install gnuplot for using this feature, please."))
			return

		sel = globs.worktable.get_selected()
		if sel == [] :
			return

		host = filter(lambda h : h == sel[0], globs.testing.hosts)[0]

		if host.out == 0 :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_WARNING,
				_("Error!"), _("At first you should make the test, please."))
			return

		h = history_graph.HistoryGraph(host)
		h.run()

	def on_menu_about(self, menu_item):
		"""
		About menu item handler.
		"""
		ad = gtk.AboutDialog()
		ad.set_transient_for(self.main_window)
		ad.set_name(preferences.const["app_name"])
		ad.set_version(preferences.const["app_version"])
		ad.set_comments(preferences.const["app_descr"])
		ad.set_website(preferences.const["app_url"])
		ad.set_authors(preferences.const["app_authors"])
		ad.set_translator_credits(preferences.const["app_translators"])
		ad.set_license(preferences.const["app_license"])
		ad.set_logo(gtk.gdk.pixbuf_new_from_file(preferences.opt["app_logo"]))
		ad.connect("response", lambda a, b : ad.destroy() )
		ad.run()

	#####
	##### gui operations
	#####
	def report_to_statusbar (self, mess, idn) :
		"""
		This function adds a message to the status bar.
		"""
		gui_funcs.gtk_threads_enter()

		stbar = self.glade_main_window.get_widget("statusbar")
		stbar.push(idn, mess)

		gui_funcs.gtk_threads_leave()

	def clear_statusbar (self, idn) :
		"""
		This function removes a message from the status bar.
		"""
		gui_funcs.gtk_threads_enter()

		stbar = self.glade_main_window.get_widget("statusbar")
		stbar.pop(idn)

		gui_funcs.gtk_threads_leave()

	def init_popup_menu(self):
		"""
		This function creates popup menu for hosts.
		"""
		self.ppmenu = self.glade_main_window.get_widget("hostmenu_")

	def set_show_test_widgets(self, state) :
		"""
		This function shows and hides several widgets.
		"""
		if state == "stop" :
			self.host_field.set_sensitive(False)
			self.testing_progress.show()

			self.glade_main_window.get_widget("play_button").hide_all()
			self.glade_main_window.get_widget("play_menu").hide_all()
			self.glade_main_window.get_widget("quickplay_menu").hide_all()

			self.glade_main_window.get_widget("pause_button").show_all()
			self.glade_main_window.get_widget("stop_button").show_all()
			self.glade_main_window.get_widget("pause_menu").show_all()
			self.glade_main_window.get_widget("stop_menu").show_all()
		elif state == "play" :
			self.host_field.set_sensitive(True)
			self.testing_progress.hide()

			self.glade_main_window.get_widget("play_button").show_all()
			self.glade_main_window.get_widget("play_menu").show_all()
			self.glade_main_window.get_widget("quickplay_menu").show_all()

			self.glade_main_window.get_widget("pause_button").hide_all()
			self.glade_main_window.get_widget("stop_button").hide_all()
			self.glade_main_window.get_widget("pause_menu").hide_all()
			self.glade_main_window.get_widget("stop_menu").hide_all()

	def set_show_host_widgets(self, state) :
		"""
		This function shows or hides the host menu in the menu bar.
		"""
		gui_funcs.gtk_threads_enter()
		self.glade_main_window.get_widget("hostmenu").set_sensitive(state)
		gui_funcs.gtk_threads_leave()

	def set_title(self, filename) :
		"""
		This function sets the window title.
		"""
		if os.name != 'nt' :
			filename = filename.replace(os.environ["HOME"], "~")

		gui_funcs.gtk_threads_enter()
		self.main_window.set_title(preferences.const["app_name"]
						+ " | " + filename)
		gui_funcs.gtk_threads_leave()

	def check_first_start(self) :
		"""
		This function suggests to customize the application.
		"""
		if os.path.exists(preferences.opt["config_user"]) :
			return
		yes = gui_funcs.show_question(self.main_window,
			(_("You have started the program %s first time. ") +\
			_("Whether you wish to customise it now?"))\
				% preferences.const["app_name"])
		if yes :
			self.show_preferences(None)

	def host_name_history_liststore_init(self) :
		self.host_name_history_liststore.clear()
		try :
			f = open(preferences.opt["host_entry_history_file"])
			for s in f :
				s = s[:-1]
				self.host_name_history_liststore.append([s])
			f.close()
		except :
			self.host_name_history_liststore.clear()

	def host_name_history_liststore_save(self) :
		try :
			f = open(preferences.opt["host_entry_history_file"], "w")
			for host in self.host_name_history_liststore :
				f.write(host[0] + "\n")
			f.close()
		except Exception(e):
			print e

	def host_name_history_liststore_add(self, host) :
		if host in map(lambda h : h[0], self.host_name_history_liststore) :
			return

		n = preferences.opt["host_entry_history_length"]
		while len(self.host_name_history_liststore) >= n :
			i = self.host_name_history_liststore.get_iter_first()
			self.host_name_history_liststore.remove(i)

		self.host_name_history_liststore.append([host])

	def configure_hosts(self) :
		"""
		This function makes automatic configuration of the hosts list.
		"""
		globs.autoconfiggui.run(self.main_window, preferences.opt["autocfg_hosts_url"])
		globs.autoconfiggui.save_list()
		if globs.autoconfiggui.canceled :
			globs.testing.clear_hosts()
		self.redraw_table(globs.testing.hosts)

	def adding_gui(self, user_data) :
		"""
		This function adds a host or a network to the main table.
		"""
		if globs.testing.pings_run :
			return False

		user_data.replace('"', '')

		# network1 is not None if user_data is 'something/n' and 1<=n<=32
		network1 = re.match("^[^/]*/([1-9]|[1-2]\d|3[0-2])$", user_data)

		# network2 is not None if user_data is 'n.n.n.n' where n is number or
		# range ('number-number')
		network2 = re.match("^%s.%s.%s.%s$" % (("\d+-?\d*",)*4), user_data)

		if network1 is not None or \
			(network2 is not None and "-" in user_data) :
			res = self.add_network_gui(user_data)
		else :
			res = self.add_host_gui(user_data)

		return res

	def add_network_gui(self, network) :
		"""
		This function adds a network to the main table.
		"""
		if not preferences.var["nmap_exist"] :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_WARNING,
				_("Error!"), _("You should to install nmap for adding a network, please."))
			return False

		self.report_to_statusbar(
			_("Getting available hosts for %s, this may take a moment. Please wait...")
			% network, 1)
		nmapobj = nmap.NMap(network)
		available_hosts = nmapobj.get_available()
		self.clear_statusbar(1)

		if available_hosts == None :
			gui_funcs.show_message(self.main_window, gtk.MESSAGE_INFO, _("Error!"),
				_("Can't get available hosts with nmap for %s.") % network)
			return False

		return reduce(lambda x,y : x or y,
				map(self.add_host_gui, available_hosts),
				False)

	def add_host_gui(self, host_name) :
		"""
		This function adds a host to the main table.
		"""
		self.main_window.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))

		self.report_to_statusbar(
			_("Make traceroute for %s, this may take a moment. Please wait...")
			% host_name, 1)
		(res, traceout) = globs.testing.add_host(host_name, "User")
		if res :
			if preferences.opt["show_traceroute_result"] :
				gui_funcs.show_message(self.main_window, gtk.MESSAGE_INFO,
					_("Traceroute's result"), traceout)
		else :
			if traceout is None :
				gui_funcs.show_message(self.main_window, gtk.MESSAGE_ERROR, _("Error!"),
					_("Can't get path to %s with traceroute.") % host_name)
			else:
				gui_funcs.show_message(self.main_window, gtk.MESSAGE_ERROR, _("Error!"),
					_("Host %s not found") % host_name)
		self.redraw_table(globs.testing.hosts)
		self.clear_statusbar(1)

		gui_funcs.gtk_threads_enter()
		self.main_window.window.set_cursor(None)
		gui_funcs.gtk_threads_leave()

		return res

	def show_window(self) :
		"""
		This function shows the application window.
		"""
		self.main_window.show()

	def hide_window(self) :
		"""
		This function hides the application window.
		"""
		self.main_window.hide()

	def get_hosts_quality(self) :
		"""
		This function finds a worst connection quality.
		"""
		q = 100
		for h in globs.testing.hosts :
			if h.enable and h.out :
				q = min(q, h.get_quality())

		return -1 if q == 100 else q

	def update_trayicon(self, q=None):
		"""
		This function updates the tray icon.
		"""
		if q is None : q = self.get_hosts_quality()
		if q != -1 :
			[globs.trayicon.set_err,
					globs.trayicon.set_warn,
					globs.trayicon.set_good][q]()
		else :
			globs.trayicon.set_good()
			globs.trayicon.remove_title()
		globs.trayicon.update_title()

