#!/usr/bin/python
# -*- coding: utf-8 -*-
##      testing.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import time

import debug
import globs
import hostentry
import ping
import traceroute
from pref import preferences

class Testing(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)
		self.clear_hosts()
		self.pings_run = False
		self.deleted_hosts = []

		self.update_after = 10

	#####
	##### list of host operations
	#####
	def clear_hosts (self) :
		"""
		This function deletes all hosts from the hosts list.
		"""
		self.hosts = []
		self.deleted_hosts = []
		self.tests_planed = 0

	def add_host(self, host_name, flag) :
		"""
		This function adds a host to testing.
		"""
		# add host with path for testing
		tr = traceroute.Traceroute(host_name)
		(host, ip_, hosts, traceout) = tr.traceroute()

		if traceout is None :
			return (False, None)

		if hosts == [] :
			return (False, "")

		for name,ip,lpath,avail,prev in hosts :
			bhe = hostentry.HostEntry(hostname=name, ip=ip)

			(exist, pos) = self.host_in_list(bhe, self.hosts)
			if exist :
				self.hosts[pos].lpath = map(
							lambda (x,y) : (x, y+1) if x == lpath else (x, y),
							self.hosts[pos].lpath
				)
			else :
				bhe.lpath = [(lpath, 1)]
				bhe.avail = avail
				bhe.previous = prev
				bhe.zone = globs.autoconfiggui.get_zonen(name, ip)
				if host == name and ip_ == ip :
					bhe.flag = flag
					bhe.traceout = traceout
				else :
					bhe.flag = ""	# TODO: get_flag(name)
					bhe.traceout = ""
				self.hosts.append(bhe)

				try :
					import GeoIP
					gi = GeoIP.new(GeoIP.GEOIP_MEMORY_CACHE)
					bhe.country_code = gi.country_code_by_addr(bhe.ip)
					bhe.country_name = gi.country_name_by_addr(bhe.ip)
				except ImportError, e:
					bhe.country_code = None
					bhe.country_name = None

			if bhe in self.deleted_hosts :
				self.deleted_hosts.remove(bhe)

		return (True, traceout)

	def host_in_list(self, host, hosts) :
		for i in range(len(hosts)) :
			if host == hosts[i] :
				return (True, i)
		return (False, -1)

	def toogle_enable_host(self, host) :
		for h in self.hosts :
			if host == h :
				h.enable = not h.enable
				break

	#####
	##### ping-test
	#####
	def pings(self, test_progress, test_end, number_of_tests) :
		"""
		This function runs ping testing.
		"""
		self.tests_planed += number_of_tests
		self.pings_run = True  # testing not stopped by user
		pings_cont = True	  # testing not ended
		pause = float(preferences.opt["ping_pause"]) / 1000
		while self.pings_run and pings_cont :
			pings_cont = False

			i = self.update_after
			k = 0
			for host in self.hosts :
				# make one ping for one host 
				if not (host.enable and\
						host.out < self.tests_planed) :
					# testing non-essential
					continue
				pings_cont = True
				k += 1

				self.make_ping(host)
				if i <= 0 :
					test_progress()
					i = self.update_after
				else :
					i -= 1
			if k <= self.update_after : test_progress()
			time.sleep(pause)

		self.pings_run = False

		preferences.const["end_date"] = time.localtime()

		test_end()

	def make_ping (self, host_entry) :
		"""
		This function runs one ping request and updates statistic for a host.
		"""
		ping_obj = ping.Ping(host_entry.ip)
		secs = ping_obj.ping()
		host_entry.out += 1
		if secs == -1 :
			host_entry.loss += 1

			host_entry.ping_history.append(0)
		else :
			if host_entry.msmax == -1. or secs > host_entry.msmax :
				host_entry.msmax = secs
			if host_entry.msmin == -1. or secs < host_entry.msmin :
				host_entry.msmin = secs
			host_entry.mssum += secs

			host_entry.ping_history.append(secs)

