#!/usr/bin/python
# -*- coding: utf-8 -*-
##      traceroute.py
##
##
## Copyright (C) 2007-2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import os
import re
import socket
import subprocess
import time

import debug
import reg

class Traceroute(debug.Debug) :
	def __init__ (self, host) :
		debug.Debug.__init__(self)

		if os.name == 'nt' :
			self.traceregexpr = "^\s*\d+.*\s+((%s)\s+.(%s|%s)|(%s|%s)).*$" \
				% (reg.host, reg.ip, reg.ip6, reg.ip, reg.ip6)
		else :
			self.traceregexpr = "^\s+\d+\s+((%s|%s|%s) \((%s|%s)\))\s+.*$" \
				% (reg.host, reg.ip, reg.ip6, reg.ip, reg.ip6)
		self.host = host

	def get_cmd(self) :
		"""
		This function returns the command for trace routing.
		"""
		if os.name == 'nt' :
			return r'..\..\..\bin\tracert "%s"' % self.host
		else :
			return 'traceroute -q1 "%s"' % self.host

	def get_scmd(self) :
		"""
		This function returns the command for trace routing (from sbin).
		"""
		return "/usr/sbin/" + self.get_cmd()

	def traceroute (self) :
		# make treaceroute for self.host
		# result:
		#	(real hostname,
		#	 ip for host,
		#	 path, list of
		#		 (hostname, ip, position in path, avail, id of previos host)
		#	 traceroute output (if traceroute not available : None),
		#	)
		time.sleep(0.2)
		try :
			(host, alises, ips) = socket.gethostbyaddr(self.host)
		except socket.herror, msg:
			if msg[0] == 0 or msg[0] == 1 :
				# Unknown host (may host has ip only)
				host = self.host
				ips = [self.host]
			elif msg[0] == 2 :
				# Host name lookup failure (problem with connection)
				# only if self.host is ip otherwise gaierror will be gotten (see below)
				# return self.host
				return (self.host, self.host,
						[(self.host, self.host, 1, False, "you")],
						""
				)
			else :
				print msg
				return ("", "", [], "")
		except socket.gaierror, msg:
			print msg
			return ("", "", [], "")
		except socket.error, msg:
			print msg
			return ("", "", [], "")

		hosts = []

		n = 0
		traceout = ""
		prev_ip = prev = "you"

		tracecmd = self.get_cmd()
		p = subprocess.Popen(self.get_cmd(), shell=True, stdout=subprocess.PIPE)

		if os.name != 'nt' :
			sts = os.waitpid(p.pid, 0)

			if sts[1] == 32512  : # command not found
				# try /usr/sbin/traceroute
				p = subprocess.Popen(self.get_scmd(), shell=True, stdout=subprocess.PIPE)
				sts = os.waitpid(p.pid, 0)

			if sts[1] != 0 :
				return ("", "", [], None)

		for strg in p.stdout :
			traceout += strg
			resexpr = re.search(self.traceregexpr, strg)
			if resexpr is not None :
				if os.name == 'nt' :
					name = resexpr.group(2)
					lastip = resexpr.group(3)
					if name is None or lastip is None :
						name = resexpr.group(4)
						lastip = resexpr.group(4)
				else :
					name = resexpr.group(2)
					lastip = resexpr.group(3)

				if name == prev :
					#or \
					#name == preferences.opt["hostname"] :
					continue
				n += 1
				hosts.append((name, lastip, n, True, prev_ip))
				prev = name
				prev_ip = name + lastip
		p.stdout.close()

		if hosts == [] or ips[0] != lastip :
			hosts.append((host, ips[0], n, False, prev_ip))

		return (host, ips[0], hosts, tracecmd + "\n\n" + traceout)

