#!/usr/bin/python
# -*- coding: utf-8 -*-
##      trayicon.py
##
##
## Copyright (C) 2010 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk

import debug
import globs
import gui_funcs
from pref import preferences

_ = gettext.gettext

class TrayIcon(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)

		self.title = ""
		self.full_title = ""

		self.tray_icon = gtk.StatusIcon()
		self.tray_icon.set_from_file(preferences.opt["app_icon"])
		self.update_title()

		self.init_menu()

		self.tray_icon.connect("activate", self.show_toogle_click)
		self.tray_icon.connect("popup-menu", self.show_menu)

	def set_good(self) :
		"""
		This function sets good status icon.
		"""
		self.tray_icon.set_from_file(preferences.opt["app_icon"])
		self.set_title(_("Connection quality is good for all hosts"))

	def set_warn(self) :
		"""
		This function sets status icon with warning icon.
		"""
		self.tray_icon.set_from_file(preferences.opt["app_icon_warn"])
		self.set_title(_("Connection quality is low-grade for some hosts"))

	def set_err(self) :
		"""
		This function sets status icon with error icon.
		"""
		self.tray_icon.set_from_file(preferences.opt["app_icon_err"])
		self.set_title(_("Connection quality is very bad for some hosts"))

	def set_title(self, title) :
		"""
		This function sets the text for the StatusIcon tooltip.
		"""
		self.title = title

	def remove_title(self) :
		"""
		This function removes the text for the StatusIcon tooltip.
		"""
		self.title = ""

	def update_title(self) :
		"""
		This function sets the StatusIcon tooltip.
		"""
		title = preferences.const["app_name"]
		if self.title :
			title += "\n" + self.title

		if self.full_title != title :
			self.tray_icon.set_tooltip(title)
			self.full_title = title

	def show_menu(self, status_icon, button, activate_time) :
		"""
		This function show popup menu.
		"""
		children = self.ppmenu.get_children()
		if globs.testing.pings_run :
			children[2].hide()
			children[3].show()
			children[4].show()
		else :
			children[2].show()
			children[3].hide()
			children[4].hide()
		self.ppmenu.popup(None, None, None, button, activate_time)

	def init_menu(self) :
		"""
		This function creates status icon menu.
		"""
		self.ppmenu = gtk.Menu()
		self.mitem_show = gtk.CheckMenuItem(_("Show %s") % preferences.const["app_name"])
		mitem_sep1  = gtk.MenuItem()
		mitem_stop  = gui_funcs.get_menu_item(gtk.STOCK_MEDIA_STOP, _("Stop test"))
		mitem_pause = gui_funcs.get_menu_item(gtk.STOCK_MEDIA_PAUSE, _("Pause test"))
		mitem_play  = gui_funcs.get_menu_item(gtk.STOCK_MEDIA_PLAY, _("Start test"))
		mitem_sep2  = gtk.MenuItem()
		mitem_quit  = gui_funcs.get_menu_item(gtk.STOCK_QUIT, _("Quit"))

											#####################
		self.ppmenu.append(self.mitem_show)	# Show/Hide window	#
		self.ppmenu.append(mitem_sep1)		# -----------------	#
		self.ppmenu.append(mitem_play)		# Start test		#
		self.ppmenu.append(mitem_stop)		# Stop test			#
		self.ppmenu.append(mitem_pause)		# Pause test		#
		self.ppmenu.append(mitem_sep2)		# -----------------	#
		self.ppmenu.append(mitem_quit)		# Quit				#
											#####################
		self.mitem_show.connect_object("activate", self.show_toogle,  None)
		mitem_quit.connect_object("activate", self.quit, None)
		mitem_play.connect_object("activate", self.start, None)
		mitem_stop.connect_object("activate", self.stop, None)
		mitem_pause.connect_object("activate", self.pause, None)

		self.mitem_show.set_active(True)

		self.ppmenu.show_all()

	def show_toogle(self, status_icon) :
		"""
		Callback for the "Show/Hide window" menu item.
		"""
		if self.mitem_show.get_active() :
			globs.testergui.show_window()
		else :
			globs.testergui.hide_window()

	def show_toogle_click(self, status_icon) :
		"""
		Callback for the "activate" signal of the status icon.
		"""
		self.mitem_show.set_active(not self.mitem_show.get_active())
		self.show_toogle(status_icon)

	def start(self, status_icon) :
		"""
		Callback for the "Start test" menu item.
		"""
		globs.testergui.on_play_clicked(None)

	def stop(self, status_icon) :
		"""
		Callback for the "Stop test" menu item.
		"""
		globs.testergui.on_stop_clicked(None)

	def pause(self, status_icon) :
		"""
		Callback for the "Pause test" menu item.
		"""
		globs.testergui.on_pause_clicked(None)

	def quit(self, status_icon) :
		"""
		Callback for the "Quit" menu item.
		"""
		globs.testergui.main_on_close(None)

