#!/usr/bin/python
# -*- coding: utf-8 -*-
##      work_table.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import gtk
import os
import string

import debug
import globs
import gui_funcs
import hostentry
from pref import preferences

_ = gettext.gettext

class WorkTable(debug.Debug) :
	def __init__ (self):
		"""
		This function forms the main table.
		"""
		debug.Debug.__init__(self)
		self.work_area_widget = None

	def set_table_widget(self, work_area_widget) :
		"""
		This function creates table for the hosts.
		"""
		self.work_area_widget = work_area_widget

		sel = self.work_area_widget.get_selection()
		sel.set_mode(gtk.SELECTION_MULTIPLE)

		if os.name != 'nt' :
			self.work_area_widget.props.has_tooltip = True
			self.work_area_widget.connect("query-tooltip", self.query_tooltip)

		self.main_liststore = gtk.ListStore(bool, gtk.gdk.Pixbuf, str, str,
				str, str, int, int, str, str, str, str, gtk.gdk.Pixbuf, int,
				str
		)
		self.work_area_widget.set_model(self.main_liststore)

		# columns
		main_tvcs = [ gtk.TreeViewColumn(""), # enable/disable
				gtk.TreeViewColumn(""), # flag
				gtk.TreeViewColumn(_("Host's name")),
				gtk.TreeViewColumn("IP"),
				gtk.TreeViewColumn(_("Quality")),
				gtk.TreeViewColumn(_("Path's length")),
				gtk.TreeViewColumn("out"),
				gtk.TreeViewColumn("loss"),
				gtk.TreeViewColumn("loss%"),
				gtk.TreeViewColumn("ms min"),
				gtk.TreeViewColumn("ms max"),
				gtk.TreeViewColumn("ms avg"),
				gtk.TreeViewColumn(_("Region")),
		]

		# dict: name of the column : the index number
		self.tvcs_dict = {
				""				: 0,
				"flag"			: 1,
				_("Host's name")	: 2,
				"IP"				: 3,
				_("Quality")		: 4,
				_("Path's length")	: 5,
				"out"			: 6,
				"loss"			: 7,
				"loss%"			: 8,
				"ms min"		: 9,
				"ms max"		: 10,
				"ms avg"		: 11,
				_("Region")		: 12,
				"zone"			: 13,
				"country_name"	: 14,
		}

		# to hide unused columns
		try :
			import GeoIP
			geoip_exists = True
		except ImportError, e:
			geoip_exists = False
		main_tvcs[self.tvcs_dict["flag"]].set_visible(geoip_exists)
		main_tvcs[self.tvcs_dict[_("Region")]].set_visible(preferences.var["prov_enable_zones"])

		# columns setting up and appending 
		for tvc in main_tvcs :
			tvc.set_sizing(gtk.TREE_VIEW_COLUMN_GROW_ONLY)
			tvc.set_resizable(True)
			tvc.set_clickable(True)
			tvc.connect("clicked", self.sort_by_column)
			self.work_area_widget.append_column(tvc)

		# cells setting up
		n = len(main_tvcs)
		main_cells = range(n)
		for i in range(n) :
			if i == self.tvcs_dict[""] or \
				i == self.tvcs_dict["flag"] or\
				i == self.tvcs_dict[_("Quality")] or\
				i == self.tvcs_dict[_("Region")] :
				continue
			main_cells[i] = gtk.CellRendererText()
			main_cells[i].set_property("editable", True)
			main_tvcs[i].pack_start(main_cells[i])
			main_tvcs[i].set_attributes(main_cells[i], text=i)

		# exceptional cells
		k = self.tvcs_dict[""]
		main_cells[k] = gtk.CellRendererToggle()
		main_cells[k].set_property("activatable", True)
		main_cells[k].connect("toggled", self.toggle_host)
		main_tvcs[k].pack_start(main_cells[k])
		main_tvcs[k].set_attributes(main_cells[k], active=k)

		k = self.tvcs_dict[_("flag")]
		main_cells[k] = gtk.CellRendererPixbuf()
		main_tvcs[k].pack_start(main_cells[k])
		main_tvcs[k].set_attributes(main_cells[k], pixbuf=k)

		k = self.tvcs_dict[_("Quality")]
		main_cells[k] = gtk.CellRendererPixbuf()
		main_tvcs[k].pack_start(main_cells[k])
		main_tvcs[k].set_attributes(main_cells[k], stock_id=k)

		k = self.tvcs_dict[_("Region")]
		main_cells[k] = gtk.CellRendererPixbuf()
		main_tvcs[k].pack_start(main_cells[k])
		main_tvcs[k].set_attributes(main_cells[k], pixbuf=k)

		# variables for query_tooltip
		self.country_column = main_tvcs[self.tvcs_dict["flag"]]
		self.quality_column = main_tvcs[self.tvcs_dict[_("Quality")]]
		self.path_column = main_tvcs[self.tvcs_dict[_("Path's length")]]
		self.region_column = main_tvcs[self.tvcs_dict[_("Region")]]

		# sorting
		for name in ["ms min", "loss%", "ms min", "ms max", "ms avg"] :
			i = self.tvcs_dict[name]
			self.main_liststore.set_sort_func(i, self.compr_float, i)

	def query_tooltip(self, widget, x, y, keyboard_tip, tooltip) :
		"""
		This function shows tooltips on a cells of the main table.
		"""
		res = widget.get_path_at_pos(x, y-20) # -20?! WTF?
		if res is None :
			return False
		(path,col,x,y) = res
		row = path[0]
		if col == self.region_column :
			z = self.main_liststore[row][self.tvcs_dict["zone"]]
			desc = (preferences.const["prov_zones"][z][0] +
					preferences.const["prov_zones"][z][1])\
							% preferences.opt["prov_name"]
		elif col == self.path_column :
			z = self.main_liststore[row][self.tvcs_dict[_("Path's length")]]
			# z = "n(m), x(y)"...
			z = map(lambda x : x.replace("(", " ").replace(")", ""), z.split(","))
			# z = ["n m", "x y", ...]
			z = map(lambda x : tuple(x.split()), z)
			# z = [("n", "m"), ("x", "y"), ...]
			z = map(lambda (x,y) :
					_("%(hops)s hop(s), it was routed %(times)s time(s)") % {"hops":x, "times":y},
					z
			)
			desc = string.join(z, ".\n")
		elif col == self.quality_column :
			z = self.main_liststore[row][self.tvcs_dict[_("Quality")]]
			if z == hostentry.q_unknown_icon :
				desc = _("Connection quality is undetermined")
			elif z == hostentry.q_good_icon :
				desc = _("Connection quality is good")
			elif z == hostentry.q_minor_icon :
				desc = _("Connection quality is low-grade")
			elif z == hostentry.q_serious_icon :
				desc = _("Connection quality is very bad")
		elif col == self.country_column :
			desc = self.main_liststore[row][self.tvcs_dict["country_name"]]
		else :
			return False

		hbox = gtk.HBox()
		label = gtk.Label(desc)
		hbox.pack_start(label, gtk.SHRINK)
		hbox.show_all()
		tooltip.set_custom(hbox)

		return True

	def toggle_host(self, CellRT, pos) :
		"""
		This function sets inclusion a host for the test.
		"""
		if self.work_area_widget is None : return

		ch = self.main_liststore[pos][self.tvcs_dict[""]]
		self.main_liststore[pos][self.tvcs_dict[""]] = not ch

		hname = self.main_liststore[pos][self.tvcs_dict[_("Host's name")]]
		hip   = self.main_liststore[pos][self.tvcs_dict["IP"]]
		host  = hostentry.HostEntry(hostname=hname, ip=hip)

		globs.testergui.toggle_selected(None, [host])

	def redraw_table_widget(self, hosts) :
		"""
		This function refills the main table.
		"""
		if self.work_area_widget is None : return

		gui_funcs.gtk_threads_enter()

		self.main_liststore.clear()
		for h in hosts :
			(msmin, msmax, msavg) = self.getms(h)

			fnamezone = preferences.const["prov_zones"][h.zone][2]
			pbufzone = gtk.gdk.pixbuf_new_from_file_at_size(fnamezone, 16, 16)

			pbuflag = None
			if h.country_code is not None :
				fnameflag = preferences.opt["earth_flags"] +\
						"/%s.png" % h.country_code.lower();
				if os.path.exists(fnameflag) :
					pbuflag = gtk.gdk.pixbuf_new_from_file(fnameflag)

			self.main_liststore.append([
					h.enable,
					pbuflag,
					h.hostname,
					h.ip,
					hostentry.q_icons[h.get_quality()],
					reduce(lambda s,(x,y): "%d(%d),%s"%(x, y, s), h.lpath, "")[:-1],
					h.out,
					h.loss,
					"%.2f" % h.clossper(),
					msmin,
					msmax,
					msavg,
					pbufzone,
					h.zone,
					h.country_name,
					])
		gui_funcs.gtk_threads_leave()

	def getms(self, h) :
		msmin = _("n/a")
		msmax = _("n/a")
		msavg = _("n/a")
		if h.msmin > 0 :
			msmin = "%.1f" % h.msmin
			msavg = "%.1f" % h.cmsavg()
		if h.msmax > 0 :
			msmax = "%.1f" % h.msmax
			msavg = "%.1f" % h.cmsavg()
		return (msmin, msmax, msavg)


	def update_table(self, hosts) :
		"""
		This function updates the cells in the main table.
		"""
		if self.work_area_widget is None : return

		for h in hosts :
			(msmin, msmax, msavg) = self.getms(h)

			j = 0
			for row in self.main_liststore :
				hname = row[self.tvcs_dict[_("Host's name")]]
				hip   = row[self.tvcs_dict["IP"]]
				if hostentry.HostEntry(hostname=hname, ip=hip) == h :
					break
				j += 1

			self.main_liststore[j][self.tvcs_dict[""]]	   = h.enable
			self.main_liststore[j][self.tvcs_dict[_("Quality")]]  =\
				hostentry.q_icons[h.get_quality()]
			self.main_liststore[j][self.tvcs_dict["out"]]	= h.out
			self.main_liststore[j][self.tvcs_dict["loss"]]   = h.loss
			self.main_liststore[j][self.tvcs_dict["loss%"]]  = "%.2f" % h.clossper()
			self.main_liststore[j][self.tvcs_dict["ms min"]] = msmin
			self.main_liststore[j][self.tvcs_dict["ms max"]] = msmax
			self.main_liststore[j][self.tvcs_dict["ms avg"]] = msavg

	def get_selected(self) :
		"""
		This function returns the list of hosts which was selected.
		"""
		if self.work_area_widget is None : return

		res = []
		s = self.work_area_widget.get_selection()
		i = self.main_liststore.get_iter_first()
		while i is not None :
			if s.iter_is_selected(i) :
				hip   = self.main_liststore.get_value(i, self.tvcs_dict["IP"])
				hname = self.main_liststore.get_value(i, self.tvcs_dict[_("Host's name")])

				host = hostentry.HostEntry(hostname=hname, ip=hip)

				res.append(host)

			i = self.main_liststore.iter_next(i)

		return res

	def sort_by_column(self, tvc) :
		"""
		This function sorts rows by the column.
		"""
		title = tvc.get_title()
		i = self.tvcs_dict[title]
		if title == _("Region") :
			i = self.tvcs_dict["zone"]

		tvc.set_sort_column_id(-1)
		tvc.set_sort_column_id(i)
		if tvc.get_sort_order() == gtk.SORT_ASCENDING and\
			tvc.get_sort_indicator() :
			tvc.set_sort_order(gtk.SORT_DESCENDING)
		else :
			tvc.set_sort_order(gtk.SORT_ASCENDING)
		tvc.set_sort_indicator(True)
		self.main_liststore.set_sort_column_id(i, tvc.get_sort_order())

	def compr_float(self, treemodel, iter1, iter2, i) :
		"""
		This function compares values in a several columns.
		"""
		try :
			num1 = float(treemodel.get_value(iter1, i))
		except :
			num1 = 0

		try :
			num2 = float(treemodel.get_value(iter2, i))
		except :
			num2 = 0

		return cmp(num1, num2)

